package gui;

import exceptions.ValidationException;
import interfaces.IModule;
import interfaces.IPanel;
import java.awt.Component;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 *
 * @author marek
 */
public class Graphic extends javax.swing.JFrame {

    public static final String PROPERTY_FILE_PATH = "/config-modules.properties";
    public static final String PROPERTY_FILE = System.getProperty("user.dir") + System.getProperty("file.separator") + "src/config-modules.properties";
    //public static final String PROPERTY_FILE = System.getProperty("user.dir") + System.getProperty("file.separator") + "config-modules.properties";
    public Properties question_modules = new Properties();
    public List<IModule> modules = new ArrayList<IModule>();
    private static int openTabs;

    /** Creates new form Graphic */
    public Graphic() {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(Graphic.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            Logger.getLogger(Graphic.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            Logger.getLogger(Graphic.class.getName()).log(Level.SEVERE, null, ex);
        } catch (UnsupportedLookAndFeelException ex) {
            Logger.getLogger(Graphic.class.getName()).log(Level.SEVERE, null, ex);
        }
        initComponents();
        setLocationRelativeTo(this);
        this.setExtendedState(this.getExtendedState() | this.MAXIMIZED_BOTH);
        openTabs = 0;

        loadModules();
    }
    /**
     * Načítanie všetkých modulov, ktoré sú zadefinované v konfiguračnom súbore.
     * Vytvorenie inštancií týchto modulov.
     */
    private void loadModules() {
        FileInputStream fis = null;
        try {
//            fis = new FileInputStream(getClass().getResource(PROPERTY_FILE_PATH).getPath());
            fis = new FileInputStream(PROPERTY_FILE);
            question_modules.load(fis);
            fis.close();
            Enumeration e = question_modules.propertyNames();
            while (e.hasMoreElements()) {
                String module_jar = question_modules.getProperty((String) e.nextElement());
                String module_class = question_modules.getProperty((String) e.nextElement());

                File file = new File(module_jar);
                URL url = file.toURI().toURL();
                URL[] urls = new URL[]{url};
                // Pridá sa nový priečinok do classpathu
//                ClassLoader loader = new URLClassLoader(urls);
                Class cls;
                try {
//                    cls = loader.loadClass(module_class);
                    cls = Class.forName(module_class);
                    final IModule module = (IModule) cls.newInstance();
                    modules.add(module);
                } catch (InstantiationException ex) {
                    Logger.getLogger(Graphic.class.getName()).log(Level.SEVERE, null, ex);
                } catch (IllegalAccessException ex) {
                    Logger.getLogger(Graphic.class.getName()).log(Level.SEVERE, null, ex);
                } catch (ClassNotFoundException ex) {
                    System.out.println("Trieda " + module_class + " nie je dostupná");
                    Logger.getLogger(Graphic.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        } catch (Exception ex) {
            System.out.println("Exception");
            Logger.getLogger(Graphic.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                fis.close();
            } catch (IOException ex) {
                Logger.getLogger(Graphic.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
    /**
     * Získanie modulu, ktorého názov je zadefinovaný v url
     * @param url
     * @return
     * @throws NullPointerException 
     */
    private IModule getModuleFromXML(String url) throws NullPointerException {
        for (int i = 0; i < modules.size(); i++) {
            if (modules.get(i).isFileAcceptable(url)) {
                return modules.get(i);
            }
        }
        throw new NullPointerException();
    }
    /**
     * Získanie XML súboru a jeho uloženie na definované miesto podľa url.
     * @param panel
     * @param url
     * @throws ValidationException 
     */
    private void saveFileXML(IPanel panel, String url) throws ValidationException {
        try {
            DOMSource source = panel.getXMLContent();
            // write the content into xml file
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            transformerFactory.setAttribute("indent-number", 1);
            Transformer transformer = transformerFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            StreamResult result = new StreamResult(new File(url));
            transformer.transform(source, result);
        } catch (TransformerException ex) {
            Logger.getLogger(Graphic.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel2 = new javax.swing.JPanel();
        mainPanel = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        jToolBar1 = new javax.swing.JToolBar();
        add_tab_button = new javax.swing.JButton();
        open_tab_button = new javax.swing.JButton();
        saveButton = new javax.swing.JButton();
        generate_xml_button = new javax.swing.JButton();
        import_moodleButton = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        main_window_tabbed_pane = new javax.swing.JTabbedPane();
        menu = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        jMenuItem1 = new javax.swing.JMenuItem();
        jMenuItem2 = new javax.swing.JMenuItem();
        jMenuItem3 = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JPopupMenu.Separator();
        jMenuItem4 = new javax.swing.JMenuItem();

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 100, Short.MAX_VALUE)
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 100, Short.MAX_VALUE)
        );

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Question Creator");

        jPanel1.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        jToolBar1.setRollover(true);

        add_tab_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/img/Add-icon.png"))); // NOI18N
        add_tab_button.setToolTipText("Nová otázka");
        add_tab_button.setFocusable(false);
        add_tab_button.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        add_tab_button.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        add_tab_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                add_tab_buttonActionPerformed(evt);
            }
        });
        jToolBar1.add(add_tab_button);

        open_tab_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/img/Open-icon.png"))); // NOI18N
        open_tab_button.setToolTipText("Otvoriť súbor");
        open_tab_button.setFocusable(false);
        open_tab_button.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        open_tab_button.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        open_tab_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                open_tab_buttonActionPerformed(evt);
            }
        });
        jToolBar1.add(open_tab_button);

        saveButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/img/Save-icon.png"))); // NOI18N
        saveButton.setToolTipText("Ulož otázku");
        saveButton.setEnabled(false);
        saveButton.setFocusable(false);
        saveButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        saveButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        saveButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(saveButton);

        generate_xml_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/img/xml.png"))); // NOI18N
        generate_xml_button.setToolTipText("Vygeneruj otázku");
        generate_xml_button.setEnabled(false);
        generate_xml_button.setFocusable(false);
        generate_xml_button.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        generate_xml_button.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        generate_xml_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                generate_xml_buttonActionPerformed(evt);
            }
        });
        jToolBar1.add(generate_xml_button);

        import_moodleButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/img/import_moodle.png"))); // NOI18N
        import_moodleButton.setToolTipText("Vytvor export Moodle");
        import_moodleButton.setFocusable(false);
        import_moodleButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        import_moodleButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        import_moodleButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                import_moodleButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(import_moodleButton);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 464, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 49, Short.MAX_VALUE)
        );

        main_window_tabbed_pane.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                main_windows_state_changed(evt);
            }
        });
        main_window_tabbed_pane.addContainerListener(new java.awt.event.ContainerAdapter() {
            public void componentAdded(java.awt.event.ContainerEvent evt) {
                main_window_tabbed_pane_ComponentAdded(evt);
            }
            public void componentRemoved(java.awt.event.ContainerEvent evt) {
                main_window_tabbed_pane_ComponentRemoved(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(main_window_tabbed_pane, javax.swing.GroupLayout.DEFAULT_SIZE, 413, Short.MAX_VALUE)
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(main_window_tabbed_pane, javax.swing.GroupLayout.DEFAULT_SIZE, 354, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addGap(30, 30, 30)
                .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGap(25, 25, 25))
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGap(29, 29, 29))
        );

        fileMenu.setText("Súbor");

        jMenuItem1.setText("Nová otázka");
        jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem1ActionPerformed(evt);
            }
        });
        fileMenu.add(jMenuItem1);

        jMenuItem2.setText("Otvoriť súbor");
        jMenuItem2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem2ActionPerformed(evt);
            }
        });
        fileMenu.add(jMenuItem2);

        jMenuItem3.setText("Vytvoriť export Moodle");
        jMenuItem3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem3ActionPerformed(evt);
            }
        });
        fileMenu.add(jMenuItem3);
        fileMenu.add(jSeparator1);

        jMenuItem4.setText("Koniec");
        jMenuItem4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem4ActionPerformed(evt);
            }
        });
        fileMenu.add(jMenuItem4);

        menu.add(fileMenu);

        setJMenuBar(menu);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(mainPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(mainPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    /**
     * Vytvorenie nového okna. Najprv za zobrazí okno na vybratie otázky.
     * @param evt 
     */
    private void add_tab_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_add_tab_buttonActionPerformed
        NewQuestion questionDialog = new NewQuestion(this, true, modules);
        questionDialog.setVisible(true);
        IModule module = questionDialog.getValue();
        if (module != null) {
            JPanel panel = module.execute(null);
            main_window_tabbed_pane.addTab(module.getName() + " " + openTabs, module.getImage(), panel);
            main_window_tabbed_pane.setTabComponentAt(main_window_tabbed_pane.getTabCount() - 1, new ButtonTabComponent(main_window_tabbed_pane));
            main_window_tabbed_pane.setSelectedIndex(main_window_tabbed_pane.getTabCount() - 1);
            openTabs++;
        }
    }//GEN-LAST:event_add_tab_buttonActionPerformed
    /**
     * Generovanie XML.
     * @param evt 
     */
    private void generate_xml_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_generate_xml_buttonActionPerformed
        IPanel panel = (IPanel) main_window_tabbed_pane.getSelectedComponent();
        JFileChooser chooser = new JFileChooser();
        FileNameExtensionFilter filter = new FileNameExtensionFilter(".xml", "xml");
        chooser.setFileFilter(filter);
        chooser.setSelectedFile(new File("default.xml"));
        int returnVal = chooser.showSaveDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            try {
                saveFileXML(panel, chooser.getSelectedFile().getAbsolutePath());
                JOptionPane.showMessageDialog(this, "XML vygenerované úspešne", "Information", JOptionPane.INFORMATION_MESSAGE);
            } catch (ValidationException ex) {
                JOptionPane.showMessageDialog(this, ex.getMessage(), "Generation", JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_generate_xml_buttonActionPerformed

    private void main_window_tabbed_pane_ComponentAdded(java.awt.event.ContainerEvent evt) {//GEN-FIRST:event_main_window_tabbed_pane_ComponentAdded
        if (!generate_xml_button.isEnabled()) {
            generate_xml_button.setEnabled(true);
        }
    }//GEN-LAST:event_main_window_tabbed_pane_ComponentAdded

    private void main_window_tabbed_pane_ComponentRemoved(java.awt.event.ContainerEvent evt) {//GEN-FIRST:event_main_window_tabbed_pane_ComponentRemoved
        if (main_window_tabbed_pane.getTabCount() == 0) {
            generate_xml_button.setEnabled(false);
        }
    }//GEN-LAST:event_main_window_tabbed_pane_ComponentRemoved
    /**
     * Otvorenie súboru. Vytvorenie okna s definíciou otázky.
     * @param evt 
     */
    private void open_tab_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_open_tab_buttonActionPerformed
        JFileChooser chooser = new JFileChooser();
        ArrayList<String> extensions = new ArrayList<String>();
        for (int i = 0; i < modules.size(); i++) {
            extensions.addAll(Arrays.asList(modules.get(i).getExtensions()));
        }
        FileNameExtensionFilter filter = new FileNameExtensionFilter("Supported files", extensions.toArray(new String[]{}));
        chooser.setFileFilter(filter);

        int returnVal = chooser.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            String url = chooser.getSelectedFile().getAbsolutePath();
            try {
                IModule module = getModuleFromXML(url);
                Document document = module.getDocumentFromFile(url);
                IPanel panel = (IPanel) module.execute(document);
                String extension = url.substring(url.lastIndexOf(".") + 1, url.length());
                if (extension.equals("xml")) {
                    main_window_tabbed_pane.addTab(panel.getUrlOfFile().substring(panel.getUrlOfFile().lastIndexOf("/") + 1, panel.getUrlOfFile().lastIndexOf(".")).replace("%20", " "), module.getImage(), (Component) panel);
                    saveButton.setEnabled(true);
                } else {
                    main_window_tabbed_pane.addTab(url.substring(url.lastIndexOf("\\") + 1, url.lastIndexOf(".")).replace("%20", " "), module.getImage(), (Component) panel);
                }
                main_window_tabbed_pane.setTabComponentAt(main_window_tabbed_pane.getTabCount() - 1, new ButtonTabComponent(main_window_tabbed_pane));
                main_window_tabbed_pane.setSelectedIndex(main_window_tabbed_pane.getTabCount() - 1);
            } catch (SAXException ex) {
                JOptionPane.showMessageDialog(this, ex.getMessage(), "XML", JOptionPane.ERROR_MESSAGE);
            } catch (NullPointerException e) {
                JOptionPane.showMessageDialog(this, "Zlý XML súbor", "XML", JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_open_tab_buttonActionPerformed

    private void saveButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveButtonActionPerformed
        IPanel panel = (IPanel) main_window_tabbed_pane.getSelectedComponent();
        try {
            saveFileXML(panel, panel.getUrlOfFile().replace("%20", " "));
        } catch (ValidationException ex) {
            JOptionPane.showMessageDialog(this, ex.getMessage(), "Saving", JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_saveButtonActionPerformed

    private void main_windows_state_changed(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_main_windows_state_changed
        IPanel panel = (IPanel) main_window_tabbed_pane.getSelectedComponent();
        if (panel != null) {
            if (panel.getUrlOfFile() != null) {
                saveButton.setEnabled(true);
            } else {
                saveButton.setEnabled(false);
            }
        } else {
            saveButton.setEnabled(false);
        }
    }//GEN-LAST:event_main_windows_state_changed
    /**
     * Vytvorenie okna pre vytvorenie exportu do systému Moodle
     * @param evt 
     */
    private void import_moodleButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_import_moodleButtonActionPerformed
        JFileChooser chooser = new JFileChooser();
        ArrayList<String> extensions = new ArrayList<String>();
        for (int i = 0; i < modules.size(); i++) {
            extensions.addAll(Arrays.asList(modules.get(i).getExtensions()));
        }
        FileNameExtensionFilter filter = new FileNameExtensionFilter("Supported files", extensions.toArray(new String[]{}));
        chooser.setFileFilter(filter);
        chooser.setMultiSelectionEnabled(true);
        int returnVal = chooser.showDialog(this, "Import");
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            HashMap<File, Document> correctFiles = new HashMap<File, Document>();
            File[] selectedFiles = chooser.getSelectedFiles();
            for (int i = 0; i < selectedFiles.length; i++) {
                try {
                    IModule module = getModuleFromXML(selectedFiles[i].getAbsolutePath());
                    correctFiles.put(selectedFiles[i], module.getDocumentFromFile(selectedFiles[i].getAbsolutePath()));
                } catch (SAXException ex) {
                    //nothing to do ... only skip wrong xml file
                } catch (NullPointerException e) {
                    //nothing to do ... only skip wrong xml file
                }
            }
            new ImportGeneration(this, true, correctFiles).setVisible(true);
        }
    }//GEN-LAST:event_import_moodleButtonActionPerformed

    private void jMenuItem4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem4ActionPerformed
        dispose();
    }//GEN-LAST:event_jMenuItem4ActionPerformed

    private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem1ActionPerformed
        add_tab_buttonActionPerformed(evt);
    }//GEN-LAST:event_jMenuItem1ActionPerformed

    private void jMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem2ActionPerformed
        open_tab_buttonActionPerformed(evt);
    }//GEN-LAST:event_jMenuItem2ActionPerformed

    private void jMenuItem3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem3ActionPerformed
        import_moodleButtonActionPerformed(evt);
    }//GEN-LAST:event_jMenuItem3ActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton add_tab_button;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JButton generate_xml_button;
    private javax.swing.JButton import_moodleButton;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JMenuItem jMenuItem3;
    private javax.swing.JMenuItem jMenuItem4;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPopupMenu.Separator jSeparator1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JTabbedPane main_window_tabbed_pane;
    private javax.swing.JMenuBar menu;
    private javax.swing.JButton open_tab_button;
    private javax.swing.JButton saveButton;
    // End of variables declaration//GEN-END:variables
}
