package it4kt.image.data;

import it4kt.image.graphic.Polygon2D;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * Trieda odpovede.
 * Odpoved obsahuje - typ odpovede, opis odpovede, zobrazeny utvar, spravnost odpovede a jedinecny kod odpovede
 * @author marek
 */
public class Answer {

    private String type;
    private Shape shape;
    private List<Integer> coords;
    private String description;
    private int code;
    private int correct;

    /**
     * Vytvara prazdnu odpoved na zadanej poziciou s vygenerovanym kodom.
     * Tento konstruktor sa vyuziva pri inicializovani(pri vytvarani novej) otazky 
     * @param position 
     */
    public Answer(String type, Shape shape, String description, int correct) {
        this.type = type;
        this.shape = shape;
        this.description = description;
        this.correct = correct;
        coords = new ArrayList<Integer>();
        if (shape.getClass().equals(Rectangle2D.Double.class)) {
            Rectangle2D.Double rect = (Rectangle2D.Double) shape;
            coords.add((int) rect.x);
            coords.add((int) rect.y);
            coords.add((int) (rect.x + rect.width));
            coords.add((int) (rect.y + rect.height));
        } else if (shape.getClass().equals(Ellipse2D.Double.class)) {
            Ellipse2D.Double ellipse = (Ellipse2D.Double) shape;
            coords.add((int) (ellipse.x + ellipse.width / 2));
            coords.add((int) (ellipse.y + ellipse.height / 2));
            coords.add((int) (ellipse.width / 2));
        } else {
            Polygon2D.Double poly = (Polygon2D.Double) shape;
            for (int i = 0; i < poly.npoints; i++) {
                coords.add((int) poly.xpoints[i]);
                coords.add((int) poly.ypoints[i]);
            }
        }
        code = new Random(System.currentTimeMillis() * coords.get(0)).nextInt(10000000);

    }

    /**
     * Vytvara odpoved. Pouziva sa vtedy ak otazka uz bola vytvorena a otavara sa zo suboru.
     * @param answer
     * @param position
     * @param code 
     */
    public Answer(String type, Shape shape, String description, int code, int correct) {
        this.type = type;
        this.code = code;
        this.shape = shape;
        this.description = description;
        this.correct = correct;
        coords = new ArrayList<Integer>();
        if (shape.getClass().equals(Rectangle2D.Double.class)) {
            Rectangle2D.Double rect = (Rectangle2D.Double) shape;
            coords.add((int) rect.x);
            coords.add((int) rect.y);
            coords.add((int) (rect.x + rect.width));
            coords.add((int) (rect.y + rect.height));
        } else if ((shape.getClass().equals(Ellipse2D.Double.class))) {
            Ellipse2D.Double ellipse = (Ellipse2D.Double) shape;
            coords.add((int) (ellipse.x + ellipse.width / 2));
            coords.add((int) (ellipse.y + ellipse.height / 2));
            coords.add((int) (ellipse.width / 2));
        } else {
            Polygon2D.Double poly = (Polygon2D.Double) shape;
            for (int i = 0; i < poly.npoints; i++) {
                coords.add((int) poly.xpoints[i]);
                coords.add((int) poly.ypoints[i]);
            }
        }
    }

    /**
     * @return the type
     */
    public String getType() {
        return type;
    }

    /**
     * @return the description
     */
    public String getDescription() {
        return description;
    }

    /**
     * @return the code
     */
    public int getCode() {
        return code;
    }

    /**
     * @return the correct
     */
    public int getCorrect() {
        return correct;
    }

    /**
     * @return the shape
     */
    public Shape getShape() {
        return shape;
    }

    /**
     * @param shape the shape to set
     */
    public void setShape(Shape shape) {
        this.shape = shape;
        coords = new ArrayList<Integer>();
        if (shape.getClass().equals(Rectangle2D.Double.class)) {
            Rectangle2D.Double rect = (Rectangle2D.Double) shape;
            coords.add((int) rect.x);
            coords.add((int) rect.y);
            coords.add((int) (rect.x + rect.width));
            coords.add((int) (rect.y + rect.height));
        } else if (shape.getClass().equals(Ellipse2D.Double.class)) {
            Ellipse2D.Double ellipse = (Ellipse2D.Double) shape;
            coords.add((int) (ellipse.x + ellipse.width / 2));
            coords.add((int) (ellipse.y + ellipse.height / 2));
            coords.add((int) (ellipse.width / 2));
        } else {
            Polygon2D.Double poly = (Polygon2D.Double) shape;
            for (int i = 0; i < poly.npoints; i++) {
                coords.add((int) poly.xpoints[i]);
                coords.add((int) poly.ypoints[i]);
            }
        }
    }

    /**
     * @return the coords
     */
    public List<Integer> getCoords() {
        return coords;
    }

    /**
     * @param description the description to set
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * @param correct the correct to set
     */
    public void setCorrect(int correct) {
        this.correct = correct;
    }

    @Override
    public String toString() {
        return "Answer " + code;
    }
}
