package it4kt.image.graphic;

import it4kt.image.graphic.panels.DesignPanelPick;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.util.ArrayList;
import java.util.List;
import javax.swing.*;
import it4kt.image.data.Answer;

public class ImageComponentPick extends JPanel {

    private DesignPanelPick parent;
    private BufferedImage image;
    private BufferedImage image_view;
    private String image_name;
    private String shape_type;
    private Point startDrag, endDrag;
    private boolean dragging = false;
    private boolean drawpoly = false;
    // Give user some leeway for selections.
    private final int PROX_DIST = 3;
    private int activeShape;
    private double scale;
    private List<Answer> answers;
    private List<Point> points;

    /**
     * Konstruktor pre image component pick
     * Nastavuje mouse listeneri. Nastavi obrazok.
     * @param originalImage
     * @param imageName 
     */
    public ImageComponentPick(BufferedImage image, String image_name, final DesignPanelPick parent) {

        this.parent = parent;
        this.image = image;
        this.image_view = image;
        this.image_name = image_name;
        answers = new ArrayList<Answer>();
        points = new ArrayList<Point>();
        activeShape = -1;
        scale = 1.0;

        setPreferredSize(new Dimension(image.getWidth(), image.getHeight()));

        addMouseListener(new MouseAdapter() {

            @Override
            public void mousePressed(MouseEvent me) {
                super.mousePressed(me);
                if (getCursor() != Cursor.getDefaultCursor()) {
                    // If cursor is set for resizing, allow dragging.
                    dragging = true;
                } else {
                    activeShape = answers.size();
                }
                startDrag = me.getPoint();
            }

            @Override
            public void mouseReleased(MouseEvent me) {
                super.mouseReleased(me);
                dragging = false;
            }

            @Override
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2 && e.getButton() == 1) {
                    points.add((Point) points.get(0).clone());
                    Polygon2D.Double poly = new Polygon2D.Double();
                    for (int i = 0; i < points.size(); i++) {
                        poly.addPoint(points.get(i).x, points.get(i).y);
                    }
                    answers.add(activeShape, new Answer("poly", poly, parent.getDescription(), parent.getCorrect()));
                    points.clear();
                    drawpoly = false;
                    parent.setSelection(answers, activeShape);
                    repaint();
                } else if (e.getClickCount() == 1 && e.getButton() == 1) {
                    for (int i = 0; i < answers.size(); i++) {
                        Rectangle r = (Rectangle) answers.get(i).getShape().getBounds().clone();
                        r.grow(PROX_DIST, PROX_DIST);
                        if (r.contains(e.getPoint())) {
                            activeShape = i;
                            setShapeType();
                            parent.setAnswer(answers.get(activeShape).getDescription(), answers.get(activeShape).getCorrect());
                            parent.setSelection(answers, activeShape);
                            break;
                        }
                        activeShape = -1;
                        parent.setSelection(answers, activeShape);
                    }
                    if (shape_type.equals("poly") && !isOverRect(e.getPoint())) {
                        drawpoly = true;
                        points.add(e.getPoint());
                    }
                } else if (e.getClickCount() == 1 && e.getButton() == 2) {
                    for (int i = 0; i < answers.size(); i++) {
                        Rectangle r = (Rectangle) answers.get(i).getShape().getBounds().clone();
                        r.grow(PROX_DIST, PROX_DIST);
                        if (r.contains(e.getPoint())) {
                            answers.remove(i);
                            activeShape = -1;
                            parent.setSelection(answers, activeShape);
                            break;
                        }
                    }
                } else if (e.getClickCount() == 1 && e.getButton() == 3) {
                    if (points.size() > 1) {
                        points.remove(points.size() - 1);
                    }
                }
                repaint();
            }
        });
        addMouseMotionListener(new MouseMotionAdapter() {

            @Override
            public void mouseDragged(MouseEvent me) {
                super.mouseDragged(me);

                if (getCursor() == Cursor.getDefaultCursor() && !shape_type.equals("poly")) {

                    // check for left mouse button
                    if ((me.getModifiersEx() & InputEvent.BUTTON1_DOWN_MASK) == 0) {
                        return;
                    }

                    // store the current location
                    endDrag = me.getPoint();
                    // calculate the size of the selection rectangle
                    double downX = startDrag.getX();
                    double downY = startDrag.getY();
                    double hereX = endDrag.getX();
                    double hereY = endDrag.getY();
                    double l = Math.min(downX, hereX);
                    double t = Math.min(downY, hereY);
                    double w = Math.abs(downX - hereX);
                    double h = Math.abs(downY - hereY);

                    if (answers.size() > activeShape) {
                        answers.remove(activeShape);
                    }
                    if (shape_type.equals("rect")) {
                        answers.add(activeShape, new Answer("rect", new Rectangle2D.Double(l, t, w, h), parent.getDescription(), parent.getCorrect()));
                    } else if (shape_type.equals("circle")) {
                        answers.add(activeShape, new Answer("circle", new Ellipse2D.Double(l, t, w, w), parent.getDescription(), parent.getCorrect()));
                    }
                    parent.setAnswer(answers.get(activeShape).getDescription(), answers.get(activeShape).getCorrect());
                    parent.setSelection(answers, activeShape);

                }
                if (dragging) {
                    Point p = me.getPoint();
                    if (shape_type.equals("rect") && activeShape != -1) {
                        Rectangle2D.Double rect = (Rectangle2D.Double) answers.get(activeShape).getShape();
                        int type = getCursor().getType();
                        int dx = (int) (p.x - rect.x);
                        int dy = (int) (p.y - rect.y);
                        switch (type) {
                            case Cursor.N_RESIZE_CURSOR:
                                int height = (int) (rect.height - dy);
                                rect.setRect(rect.x, rect.y + dy, rect.width, height);
                                break;
                            case Cursor.NW_RESIZE_CURSOR:
                                int width = (int) (rect.width - dx);
                                height = (int) (rect.height - dy);
                                rect.setRect(rect.x + dx, rect.y + dy, width, height);
                                break;
                            case Cursor.W_RESIZE_CURSOR:
                                width = (int) (rect.width - dx);
                                rect.setRect(rect.x + dx, rect.y, width, rect.height);
                                break;
                            case Cursor.SW_RESIZE_CURSOR:
                                width = (int) (rect.width - dx);
                                height = dy;
                                rect.setRect(rect.x + dx, rect.y, width, height);
                                break;
                            case Cursor.S_RESIZE_CURSOR:
                                height = dy;
                                rect.setRect(rect.x, rect.y, rect.width, height);
                                break;
                            case Cursor.SE_RESIZE_CURSOR:
                                width = dx;
                                height = dy;
                                rect.setRect(rect.x, rect.y, width, height);
                                break;
                            case Cursor.E_RESIZE_CURSOR:
                                width = dx;
                                rect.setRect(rect.x, rect.y, width, rect.height);
                                break;
                            case Cursor.NE_RESIZE_CURSOR:
                                width = dx;
                                height = (int) (rect.height - dy);
                                rect.setRect(rect.x, rect.y + dy, width, height);
                                break;
                            case Cursor.MOVE_CURSOR:
                                int move_x = p.x - startDrag.x;
                                int move_y = p.y - startDrag.y;
                                rect.setRect(rect.x + move_x, rect.y + move_y, rect.width, rect.height);
                                startDrag = new Point(startDrag.x + move_x, startDrag.y + move_y);
                                break;
                            default:
                                System.out.println("unexpected type: " + type);
                        }

                    } else if (shape_type.equals("circle") && activeShape != -1) {
                        Ellipse2D.Double ellipse = (Ellipse2D.Double) answers.get(activeShape).getShape();
                        int type = getCursor().getType();
                        int dx = (int) (p.x - ellipse.x);
                        int dy = (int) (p.y - ellipse.y);
                        switch (type) {
                            case Cursor.N_RESIZE_CURSOR:
                                int height = (int) (ellipse.height - dy);
                                ellipse = new Ellipse2D.Double(ellipse.x, ellipse.y + dy, height, height);
                                break;
                            case Cursor.NW_RESIZE_CURSOR:
                                int width = (int) (ellipse.width - dx);
                                height = (int) (ellipse.height - dy);
                                ellipse = new Ellipse2D.Double(ellipse.x + dx, ellipse.y + dy, width, width);
                                break;
                            case Cursor.W_RESIZE_CURSOR:
                                width = (int) (ellipse.width - dx);
                                ellipse = new Ellipse2D.Double(ellipse.x + dx, ellipse.y, width, width);
                                break;
                            case Cursor.SW_RESIZE_CURSOR:
                                width = (int) (ellipse.width - dx);
                                height = dy;
                                ellipse = new Ellipse2D.Double(ellipse.x + dx, ellipse.y, width, width);
                                break;
                            case Cursor.S_RESIZE_CURSOR:
                                height = dy;
                                ellipse = new Ellipse2D.Double(ellipse.x, ellipse.y, height, height);
                                break;
                            case Cursor.SE_RESIZE_CURSOR:
                                width = dx;
                                height = dy;
                                ellipse = new Ellipse2D.Double(ellipse.x, ellipse.y, width, width);
                                break;
                            case Cursor.E_RESIZE_CURSOR:
                                width = dx;
                                ellipse = new Ellipse2D.Double(ellipse.x, ellipse.y, width, width);
                                break;
                            case Cursor.NE_RESIZE_CURSOR:
                                width = dx;
                                height = (int) (ellipse.height - dy);
                                ellipse = new Ellipse2D.Double(ellipse.x, ellipse.y + dy, width, width);
                                break;
                            case Cursor.MOVE_CURSOR:
                                int move_x = p.x - startDrag.x;
                                int move_y = p.y - startDrag.y;
                                ellipse = new Ellipse2D.Double(ellipse.x + move_x, ellipse.y + move_y, ellipse.width, ellipse.height);
                                startDrag = new Point(startDrag.x + move_x, startDrag.y + move_y);
                                break;
                            default:
                                System.out.println("unexpected type: " + type);
                        }
                        answers.get(activeShape).setShape(ellipse);
                    } else if (shape_type.equals("poly") && activeShape != -1) {
                        Polygon2D.Double poly = (Polygon2D.Double) answers.get(activeShape).getShape();
                        int type = getCursor().getType();
                        switch (type) {
                            case Cursor.MOVE_CURSOR:
                                int move_x = p.x - startDrag.x;
                                int move_y = p.y - startDrag.y;
                                poly.translate(move_x, move_y);
                                startDrag = new Point(startDrag.x + move_x, startDrag.y + move_y);
                                break;
                            default:
                                System.out.println("unexpected type: " + type);
                        }
                        answers.get(activeShape).setShape(poly);
                    }
                }
                // queue a repaint of the panel
                repaint();
            }

            @Override
            public void mouseMoved(MouseEvent e) {
                Point p = e.getPoint();

                if (!isOverRect(p)) {
                    if (getCursor() != Cursor.getDefaultCursor()) {
                        // If cursor is not over rect reset it to the default.
                        setCursor(Cursor.getDefaultCursor());
                    }
                    return;
                }

                if (shape_type.equals("poly") && isOverRect(p)) {
                    setCursor(Cursor.getPredefinedCursor(
                            Cursor.MOVE_CURSOR));
                    return;
                }

                // Locate cursor relative to center of rect.
                int outcode = getOutcode(p);
                Rectangle rect = answers.get(activeShape).getShape().getBounds();
                switch (outcode) {
                    case Rectangle.OUT_TOP:
                        if (Math.abs(p.y - rect.y) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.N_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_TOP + Rectangle.OUT_LEFT:
                        if (Math.abs(p.y - rect.y) < PROX_DIST
                                && Math.abs(p.x - rect.x) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.NW_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_LEFT:
                        if (Math.abs(p.x - rect.x) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.W_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_LEFT + Rectangle.OUT_BOTTOM:
                        if (Math.abs(p.x - rect.x) < PROX_DIST
                                && Math.abs(p.y - (rect.y + rect.height)) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.SW_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_BOTTOM:
                        if (Math.abs(p.y - (rect.y + rect.height)) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.S_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_BOTTOM + Rectangle.OUT_RIGHT:
                        if (Math.abs(p.x - (rect.x + rect.width)) < PROX_DIST
                                && Math.abs(p.y - (rect.y + rect.height)) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.SE_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_RIGHT:
                        if (Math.abs(p.x - (rect.x + rect.width)) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.E_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_RIGHT + Rectangle.OUT_TOP:
                        if (Math.abs(p.x - (rect.x + rect.width)) < PROX_DIST
                                && Math.abs(p.y - rect.y) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.NE_RESIZE_CURSOR));
                        }
                        break;
                    default:    // center
                        setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
                }
            }
        });

    }

    /**
     * Make a smaller Rectangle and use it to locate the
     * cursor relative to the Rectangle center.
     */
    private int getOutcode(Point p) {
        Rectangle r = (Rectangle) answers.get(activeShape).getShape().getBounds().clone();
        r.grow(-PROX_DIST, -PROX_DIST);
        return r.outcode(p.x, p.y);
    }

    /**
     * Make a larger Rectangle and check to see if the
     * cursor is over it.
     */
    private boolean isOverRect(Point p) {
        if (activeShape != -1 && answers.size() > activeShape) {
            Rectangle r = (Rectangle) answers.get(activeShape).getShape().getBounds().clone();
            r.grow(PROX_DIST, PROX_DIST);
            return r.contains(p);
        }
        return false;
    }

    @Override
    public void paintComponent(Graphics g2D) {
        Graphics2D g = (Graphics2D) g2D;
        g.drawImage(image_view, 0, 0, null);
        if (drawpoly) {
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            g.setPaint(Color.BLACK);
            for (int j = 0; j < points.size() - 1; j++) {
                g.draw(new Line2D.Double(points.get(j), points.get(j + 1)));
            }
            g.setPaint(Color.YELLOW);
            for (int j = 0; j < points.size(); j++) {
                Point p = points.get(j);
                g.fill(new Ellipse2D.Double(p.x - 2, p.y - 2, 4, 4));
            }
        }
        for (int i = 0; i < answers.size(); i++) {
            if (activeShape != i) {
                drawShape(g, Color.BLUE, Color.WHITE, i);
            } else {
                drawShape(g, Color.YELLOW, Color.BLACK, i);
                Rectangle rect = answers.get(i).getShape().getBounds();

                AlphaComposite ta = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.1f);
                g.setComposite(ta);
                g.setComposite(AlphaComposite.SrcOver);
                g.setColor(Color.RED);
                g.draw(rect);

                g.setPaint(Color.RED);
                g.fill(new Rectangle.Double(rect.getX() - 3, rect.getY() - 3, 6, 6));
                g.fill(new Rectangle.Double(rect.getCenterX() - 3, rect.getY() - 3, 6, 6));
                g.fill(new Rectangle.Double(rect.getX() + rect.getWidth() - 3, rect.getY() - 3, 6, 6));

                g.fill(new Rectangle.Double(rect.getX() + rect.getWidth() - 3, rect.getCenterY() - 3, 6, 6));
                g.fill(new Rectangle.Double(rect.getX() + rect.getWidth() - 3, rect.getY() + rect.getHeight() - 3, 6, 6));

                g.fill(new Rectangle.Double(rect.getCenterX() - 3, rect.getY() + rect.getHeight() - 3, 6, 6));
                g.fill(new Rectangle.Double(rect.getX() - 3, rect.getY() + rect.getHeight() - 3, 6, 6));

                g.fill(new Rectangle.Double(rect.getX() - 3, rect.getCenterY() - 3, 6, 6));

            }
        }

    }

    /**
     * Nakreslenie utvaru odpovede
     * @param g
     * @param shape_fill
     * @param shape_border
     * @param shape_index 
     */
    public void drawShape(Graphics2D g, Color shape_fill, Color shape_border, int shape_index) {
        AlphaComposite ta = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.1f);
        g.setComposite(ta);
        g.setColor(shape_fill);
        g.fill(answers.get(shape_index).getShape());
        g.setComposite(AlphaComposite.SrcOver);
        g.setColor(shape_border);
        g.draw(answers.get(shape_index).getShape());
    }

    /**
     * pridanie odpovede
     * @param answer 
     */
    public void addSelection(Answer answer) {
        answers.add(answer);
        parent.setSelection(answers, activeShape);
        repaint();
    }

    /**
     * vrati odpovede
     * @return 
     */
    public List<Answer> getSelection() {
        for (int i = 0; i < answers.size(); i++) {
            if (answers.get(i).getShape().getClass().equals(Rectangle2D.Double.class)) {
                Rectangle2D.Double rect = (Rectangle2D.Double) answers.get(i).getShape();
                rect.setRect(rect.x / scale, rect.y / scale, rect.width / scale, rect.height / scale);
                answers.get(i).setShape(rect);
            } else if ((answers.get(i).getShape().getClass().equals(Ellipse2D.Double.class))) {
                Ellipse2D.Double ellipse = (Ellipse2D.Double) answers.get(i).getShape();
                ellipse = new Ellipse2D.Double(ellipse.x / scale, ellipse.y / scale, ellipse.width / scale, ellipse.height / scale);
                answers.get(i).setShape(ellipse);
            } else {
                Polygon2D.Double poly = (Polygon2D.Double) answers.get(i).getShape();
                Polygon2D.Double poly2 = new Polygon2D.Double();
                for (int j = 0; j < poly.npoints; j++) {
                    poly2.addPoint(poly.xpoints[j] / scale, poly.ypoints[j] / scale);
                }
                answers.get(i).setShape(poly2);
            }
        }
        return answers;
    }

    /**
     * Nastavenie odpovedi 
     */
    public void setSelection() {
        for (int i = 0; i < answers.size(); i++) {
            if (answers.get(i).getShape().getClass().equals(Rectangle2D.Double.class)) {
                Rectangle2D.Double rect = (Rectangle2D.Double) ((Rectangle2D.Double) answers.get(i).getShape()).clone();
                rect.setRect(rect.x * scale, rect.y * scale, rect.width * scale, rect.height * scale);
                answers.get(i).setShape(rect);
            } else if (answers.get(i).getShape().getClass().equals(Ellipse2D.Double.class)) {
                Ellipse2D.Double ellipse = (Ellipse2D.Double) ((Ellipse2D.Double) answers.get(i).getShape()).clone();
                ellipse = new Ellipse2D.Double(ellipse.x * scale, ellipse.y * scale, ellipse.width * scale, ellipse.height * scale);
                answers.get(i).setShape(ellipse);
            } else {
                Polygon2D.Double poly = (Polygon2D.Double) answers.get(i).getShape();
                Polygon2D.Double poly2 = new Polygon2D.Double();
                for (int j = 0; j < poly.npoints; j++) {
                    poly2.addPoint(poly.xpoints[j] * scale, poly.ypoints[j] * scale);
                }
                answers.get(i).setShape(poly2);
            }
        }
    }

    /**
     * Nastavenie aktualne upravovanej odpoved podla pozicie v zozname
     * @param activeShape 
     */
    public void setSelection(int activeShape) {
        this.activeShape = activeShape;
        setShapeType();
        parent.setAnswer(answers.get(activeShape).getDescription(), answers.get(activeShape).getCorrect());
        repaint();
    }

    /**
     * nastavenie typu tvaru, ktoru ma odpoved
     */
    private void setShapeType() {
        if (activeShape != -1) {
            if (answers.get(activeShape).getShape().getClass().equals(Rectangle2D.Double.class)) {
                shape_type = "rect";
            } else if (answers.get(activeShape).getShape().getClass().equals(Ellipse2D.Double.class)) {
                shape_type = "circle";
            } else {
                shape_type = "poly";
            }
            parent.setShape(shape_type);
        }
    }

    /**
     * Odstranenie odpovede
     * @param index 
     */
    public void removeSelection(int index) {
        if (index == -1) {
            return;
        }
        this.activeShape = -1;
        if (answers.size() > index) {
            answers.remove(index);
        }
        parent.setSelection(answers, activeShape);
        repaint();
    }

    /**
     * Priblizuje alebo oddialuje obrazok
     * @param curr_scale
     * @param next 
     */
    public void zoom(double curr_scale, double next) {
        this.scale = curr_scale + next;
        int width = (int) (scale * image.getWidth());
        int height = (int) (scale * image.getHeight());
        image_view = scaleImage(width, height);
        for (int i = 0; i < answers.size(); i++) {
            if (answers.get(i).getShape().getClass().equals(Rectangle2D.Double.class)) {
                Rectangle2D.Double rect = (Rectangle2D.Double) answers.get(i).getShape();
                rect.setRect(rect.x * scale / curr_scale, rect.y * scale / curr_scale, rect.width * scale / curr_scale, rect.height * scale / curr_scale);
                answers.get(i).setShape(rect);
            } else if (answers.get(i).getShape().getClass().equals(Ellipse2D.Double.class)) {
                Ellipse2D.Double ellipse = (Ellipse2D.Double) answers.get(i).getShape();
                ellipse = new Ellipse2D.Double(ellipse.x * scale / curr_scale, ellipse.y * scale / curr_scale, ellipse.width * scale / curr_scale, ellipse.height * scale / curr_scale);
                answers.get(i).setShape(ellipse);
            } else {
                Polygon2D.Double poly = (Polygon2D.Double) answers.get(i).getShape();
                Polygon2D.Double poly2 = new Polygon2D.Double();
                for (int j = 0; j < poly.npoints; j++) {
                    poly2.addPoint(poly.xpoints[j] * scale / curr_scale, poly.ypoints[j] * scale / curr_scale);
                }
                answers.get(i).setShape(poly2);
            }
        }
    }

    /**
     * Zmena rozmerov obrazku.
     * @param dim 
     */
    public void imageResize(Dimension dim) {
        image_view = scaleImage((int) dim.getWidth(), (int) dim.getHeight());
        image = image_view;
        answers.clear();
        activeShape = -1;
    }

    /**
     * Zmena velkosti obrazka
     * @param width
     * @param height
     * @return 
     */
    public BufferedImage scaleImage(int width, int height) {
        BufferedImage image_scale = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        Graphics2D g = image_scale.createGraphics();
        AffineTransform at =
                AffineTransform.getScaleInstance((double) width / image.getWidth(),
                (double) height / image.getHeight());
        g.drawRenderedImage(image, at);
        setPreferredSize(new Dimension(width, height));
        setSize(new Dimension(width, height));
        return image_scale;
    }

    /**
     * @return the imageName
     */
    public String getImageName() {
        return image_name;
    }

    /**
     * Vrati obrazok
     * @return 
     */
    public BufferedImage getImage() {
        return image;
    }

    /**
     * @param shapeType the shapeType to set
     */
    public void setShapeType(String shape_type) {
        this.shape_type = shape_type;
        points.clear();
    }

    /**
     * Nastavenie popisu odpovede
     * @param description 
     */
    public void setDescription(String description) {
        if (activeShape != -1) {
            answers.get(activeShape).setDescription(description);
        }
    }

    /**
     * nastavenie ci je odpoved spravna alebo nie, ak je nastaveni singleselect potom sa ostatne odpovede nastavia na zle.
     * @param correct
     * @param isSingleSelect 
     */
    public void setCorrect(int correct, boolean isSingleSelect) {
        if (activeShape != -1) {
            if (isSingleSelect) {
                for (int i = 0; i < answers.size(); i++) {
                    if (i != activeShape) {
                        answers.get(i).setCorrect(-1);
                    }
                }
                answers.get(activeShape).setCorrect(correct);
            } else {
                answers.get(activeShape).setCorrect(correct);
            }
        }
    }
}
