package it4kt.image.graphic;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import javax.swing.*;

public class ImageComponentSelectArea extends JPanel {

    private BufferedImage originalImage;
    private BufferedImage viewedImage;
    private String imageName;
    Rectangle2D.Double selectionRect;
    Point mouseDown, mouseUp;
    private double scale;
    boolean dragging = false;
    // Give user some leeway for selections.
    final int PROX_DIST = 3;

    /**
     * Konstruktor pre image component select area
     * Nastavuje mouse listeneri. Nastavi obrazok.
     * @param originalImage
     * @param imageName 
     */
    public ImageComponentSelectArea(BufferedImage originalImage, String imageName) {
        this.originalImage = originalImage;
        this.imageName = imageName;
        viewedImage = originalImage;
        scale = 1.0;

        setPreferredSize(new Dimension(originalImage.getWidth(), originalImage.getHeight()));
        addMouseListener(new MouseAdapter() {

            public void mousePressed(MouseEvent me) {
                super.mousePressed(me);
                if (getCursor() != Cursor.getDefaultCursor()) {
                    // If cursor is set for resizing, allow dragging.
                    dragging = true;
                }
                mouseDown = me.getPoint();
            }

            public void mouseReleased(MouseEvent me) {
                super.mouseReleased(me);
                dragging = false;
            }

            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    selectionRect = null;
                    repaint();
                }
            }
        });
        addMouseMotionListener(new MouseMotionAdapter() {

            public void mouseDragged(MouseEvent me) {
                super.mouseDragged(me);
                if (getCursor() == Cursor.getDefaultCursor()) {
                    // check for left mouse button
                    if ((me.getModifiersEx() & InputEvent.BUTTON1_DOWN_MASK) == 0) {
                        return;
                    }

                    // store the current location
                    mouseUp = me.getPoint();
                    if (me.getPoint().getX() < 0) {
                        mouseUp = new Point(-4, (int) me.getPoint().getY());
                    }
                    if (me.getPoint().getY() < 0) {
                        mouseUp = new Point((int) mouseUp.getX(), -4);
                    }
                    if (me.getPoint().getX() > viewedImage.getWidth()) {
                        mouseUp = new Point(viewedImage.getWidth() - 4, (int) mouseUp.getY());
                    }
                    if (me.getPoint().getY() > viewedImage.getHeight()) {
                        mouseUp = new Point((int) mouseUp.getX(), viewedImage.getHeight() - 4);
                    }
                    // calculate the size of the selection rectangle
                    double downX = mouseDown.getX();
                    double downY = mouseDown.getY();
                    double hereX = mouseUp.getX();
                    double hereY = mouseUp.getY();
                    double l = Math.min(downX, hereX);
                    double t = Math.min(downY, hereY);
                    double w = Math.abs(downX - hereX);
                    double h = Math.abs(downY - hereY);
                    selectionRect = new Rectangle2D.Double(l, t, w, h);
                }
                if (dragging) {
                    Point p = me.getPoint();

                    int type = getCursor().getType();
                    int dx = (int) (p.x - selectionRect.x);
                    int dy = (int) (p.y - selectionRect.y);
                    switch (type) {
                        case Cursor.N_RESIZE_CURSOR:
                            int height = (int) (selectionRect.height - dy);
                            selectionRect.setRect(selectionRect.x, selectionRect.y + dy, selectionRect.width, height);
                            break;
                        case Cursor.NW_RESIZE_CURSOR:
                            int width = (int) (selectionRect.width - dx);
                            height = (int) (selectionRect.height - dy);
                            selectionRect.setRect(selectionRect.x + dx, selectionRect.y + dy, width, height);
                            break;
                        case Cursor.W_RESIZE_CURSOR:
                            width = (int) (selectionRect.width - dx);
                            selectionRect.setRect(selectionRect.x + dx, selectionRect.y, width, selectionRect.height);
                            break;
                        case Cursor.SW_RESIZE_CURSOR:
                            width = (int) (selectionRect.width - dx);
                            height = dy;
                            selectionRect.setRect(selectionRect.x + dx, selectionRect.y, width, height);
                            break;
                        case Cursor.S_RESIZE_CURSOR:
                            height = dy;
                            selectionRect.setRect(selectionRect.x, selectionRect.y, selectionRect.width, height);
                            break;
                        case Cursor.SE_RESIZE_CURSOR:
                            width = dx;
                            height = dy;
                            selectionRect.setRect(selectionRect.x, selectionRect.y, width, height);
                            break;
                        case Cursor.E_RESIZE_CURSOR:
                            width = dx;
                            selectionRect.setRect(selectionRect.x, selectionRect.y, width, selectionRect.height);
                            break;
                        case Cursor.NE_RESIZE_CURSOR:
                            width = dx;
                            height = (int) (selectionRect.height - dy);
                            selectionRect.setRect(selectionRect.x, selectionRect.y + dy, width, height);
                            break;
                        case Cursor.MOVE_CURSOR:
                            int move_x = p.x - mouseDown.x;
                            int move_y = p.y - mouseDown.y;
                            selectionRect.setRect(selectionRect.x + move_x, selectionRect.y + move_y, selectionRect.width, selectionRect.height);
                            mouseDown = new Point(mouseDown.x + move_x, mouseDown.y + move_y);
                            break;
                        default:
                            System.out.println("unexpected type: " + type);
                    }
                }
                // queue a repaint of the panel
                repaint();
            }

            public void mouseMoved(MouseEvent e) {
                Point p = e.getPoint();
                if (!isOverRect(p)) {
                    if (getCursor() != Cursor.getDefaultCursor()) {
                        // If cursor is not over rect reset it to the default.
                        setCursor(Cursor.getDefaultCursor());
                    }
                    return;
                }
                // Locate cursor relative to center of rect.
                int outcode = getOutcode(p);

                switch (outcode) {
                    case Rectangle.OUT_TOP:
                        if (Math.abs(p.y - selectionRect.y) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.N_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_TOP + Rectangle.OUT_LEFT:
                        if (Math.abs(p.y - selectionRect.y) < PROX_DIST
                                && Math.abs(p.x - selectionRect.x) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.NW_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_LEFT:
                        if (Math.abs(p.x - selectionRect.x) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.W_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_LEFT + Rectangle.OUT_BOTTOM:
                        if (Math.abs(p.x - selectionRect.x) < PROX_DIST
                                && Math.abs(p.y - (selectionRect.y + selectionRect.height)) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.SW_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_BOTTOM:
                        if (Math.abs(p.y - (selectionRect.y + selectionRect.height)) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.S_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_BOTTOM + Rectangle.OUT_RIGHT:
                        if (Math.abs(p.x - (selectionRect.x + selectionRect.width)) < PROX_DIST
                                && Math.abs(p.y - (selectionRect.y + selectionRect.height)) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.SE_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_RIGHT:
                        if (Math.abs(p.x - (selectionRect.x + selectionRect.width)) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.E_RESIZE_CURSOR));
                        }
                        break;
                    case Rectangle.OUT_RIGHT + Rectangle.OUT_TOP:
                        if (Math.abs(p.x - (selectionRect.x + selectionRect.width)) < PROX_DIST
                                && Math.abs(p.y - selectionRect.y) < PROX_DIST) {
                            setCursor(Cursor.getPredefinedCursor(
                                    Cursor.NE_RESIZE_CURSOR));
                        }
                        break;
                    default:    // center
                        setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
                }
            }
        });
    }

    /**
     * Make a smaller Rectangle and use it to locate the
     * cursor relative to the Rectangle center.
     */
    private int getOutcode(Point p) {
        Rectangle r = (Rectangle) selectionRect.getBounds().clone();
        r.grow(-PROX_DIST, -PROX_DIST);
        return r.outcode(p.x, p.y);
    }

    /**
     * Make a larger Rectangle and check to see if the
     * cursor is over it.
     */
    private boolean isOverRect(Point p) {
        if (selectionRect != null) {
            Rectangle r = (Rectangle) selectionRect.getBounds().clone();
            r.grow(PROX_DIST, PROX_DIST);
            return r.contains(p);
        }
        return false;
    }

    /**
     * Metoda, ktora zobrazuje obrazok a odpoved
     * @param g2D 
     */
    public void paintComponent(Graphics g2D) {
        Graphics2D g = (Graphics2D) g2D;
        g.drawImage(viewedImage, 0, 0, null);
        if (selectionRect != null) {
            AlphaComposite ta = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.1f);
            g.setComposite(ta);
            g.setColor(Color.BLUE);
            g.fill(selectionRect);
            g.setComposite(AlphaComposite.SrcOver);
            g.setColor(Color.GRAY);
            g.draw(selectionRect);
        }
    }

    /**
     * Nastavuje odpoved (tvar odpovede - stvorec)
     * @param cx
     * @param cy
     * @param width
     * @param height 
     */
    public void setSelection(int cx, int cy, int width, int height) {
        selectionRect = new Rectangle2D.Double(cx - width / 2, cy - height / 2, width, height);
        repaint();
    }

    /**
     * Nastavuje odpoved (tvar odpovede - stvorec)
     * @param x
     * @param y
     * @param width
     * @param height 
     */
    public void setSelection(double x, double y, double width, double height) {
        selectionRect = new Rectangle2D.Double(x, y, width, height);
        repaint();
    }

    /**
     * Priblizuje alebo oddialuje obrazok
     * @param curr_scale
     * @param next 
     */
    public void zoom(double curr_scale, double next) {
        this.scale = curr_scale + next;
        if (scale == 1.0) {
            viewedImage = originalImage;
            setPreferredSize(new Dimension(originalImage.getWidth(), originalImage.getHeight()));
            setSize(new Dimension(originalImage.getWidth(), originalImage.getHeight()));
            if (selectionRect != null) {
                setSelection(selectionRect.x * scale / curr_scale, selectionRect.y * scale / curr_scale, scale * selectionRect.width / curr_scale, scale * selectionRect.height / curr_scale);
            }
        } else {
            int width = (int) (scale * originalImage.getWidth());
            int height = (int) (scale * originalImage.getHeight());
            BufferedImage zoomedImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
            Graphics2D g = zoomedImage.createGraphics();
            AffineTransform at =
                    AffineTransform.getScaleInstance((double) width / originalImage.getWidth(),
                    (double) height / originalImage.getHeight());
            g.drawRenderedImage(originalImage, at);
            viewedImage = zoomedImage;
            setPreferredSize(new Dimension(width, height));
            setSize(new Dimension(width, height));
            if (selectionRect != null) {
                setSelection(selectionRect.x * scale / curr_scale, selectionRect.y * scale / curr_scale, scale * selectionRect.width / curr_scale, scale * selectionRect.height / curr_scale);
            }
        }
    }

    /**
     * Zmena rozmerov obrazku.
     * @param dim 
     */
    public void imageResize(Dimension dim) {
        BufferedImage resizedImage = new BufferedImage((int) dim.getWidth(), (int) dim.getHeight(), BufferedImage.TYPE_INT_RGB);
        Graphics2D g = resizedImage.createGraphics();
        AffineTransform at =
                AffineTransform.getScaleInstance((double) dim.getWidth() / originalImage.getWidth(),
                (double) dim.getHeight() / originalImage.getHeight());
        g.drawRenderedImage(originalImage, at);
        viewedImage = resizedImage;
        originalImage = resizedImage;
        setPreferredSize(dim);
        setSize(dim);
        selectionRect = null;
    }

    /**
     * vrati odpoved
     * @return 
     */
    public Rectangle2D.Double getSelection() {
        return new Rectangle2D.Double((selectionRect.x + selectionRect.width / 2) / scale, (selectionRect.y + selectionRect.height / 2) / scale, selectionRect.width / scale, selectionRect.height / scale);
    }

    /**
     * @return the imageName
     */
    public String getImageName() {
        return imageName;
    }

    /**
     * Vrati obrazok
     * @return 
     */
    public BufferedImage getImage() {
        return originalImage;
    }
}
