/*
 * DesignPanel.java
 *
 * Created on 20.11.2011, 22:16:17
 */
package it4kt.image.graphic.panels;

import exceptions.ValidationException;
import it4kt.image.graphic.ImageComponentSelectArea;
import it4kt.image.graphic.dialogs.ResizeDialog;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.DefaultListModel;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.filechooser.FileNameExtensionFilter;
import it4kt.image.data.SelectAreaDefinition;
import org.apache.commons.codec.binary.Base64;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Tento panel sa vyuziva na graficku tvorbu otazky (mbimage) select area.
 * @author marek
 */
public class DesignPanelSelectArea extends javax.swing.JPanel {

    DefaultListModel model;
    ImageComponentSelectArea imageComponent;

    /** Creates new form DesignPanel 
     * Konstruktor triedy DesignPanel. Vytvoria a nastavia sa vsetky preddefinovane hodnoty. 
     * Ak je zadany vstupny subor potom sa analyzuje a vytvori sa jeho graficka reprezentacia.
     */
    public DesignPanelSelectArea(Document document) {
        initComponents();
        initQuestion(document);
    }

    /**
     * Metoda nastavuje hodnoty pre otazku mbimage selectarea
     * Ak dokument existuje potom sa nastavia hodnoty otazky z tohto dokumentu.
     * Ak dokument neexistuje potom sa nastavia preddefinovane hodnoty.
     * @param document 
     */
    private void initQuestion(Document document) {
        if (document != null) {

            Element root = document.getDocumentElement();
            if (root.getElementsByTagName("name").item(0).getChildNodes().getLength() != 0) {
                questionNameTextField.setText(root.getElementsByTagName("name").item(0).getChildNodes().item(0).getNodeValue());
            }
            if (root.getElementsByTagName("questiontext").item(0).getChildNodes().getLength() != 0) {
                textOtazkyTextArea.setText(root.getElementsByTagName("questiontext").item(0).getChildNodes().item(0).getNodeValue());
            }
            try {
                imagePanel.removeAll();
                byte[] imageBytes = Base64.decodeBase64(root.getElementsByTagName("image").item(0).getChildNodes().item(0).getNodeValue());
                InputStream in = new ByteArrayInputStream(imageBytes);
                BufferedImage image = ImageIO.read(in);
                imageComponent = new ImageComponentSelectArea(image, root.getElementsByTagName("image").item(0).getAttributes().getNamedItem("name").getChildNodes().item(0).getNodeValue());
                imageComponent.setSize(image.getWidth(), image.getHeight());
                insertImage();
                jTextField1.setText("100");
                imageComponent.setSelection(Integer.parseInt(root.getElementsByTagName("x").item(0).getChildNodes().item(0).getNodeValue()), Integer.parseInt(root.getElementsByTagName("y").item(0).getChildNodes().item(0).getNodeValue()), Integer.parseInt(root.getElementsByTagName("width").item(0).getChildNodes().item(0).getNodeValue()), Integer.parseInt(root.getElementsByTagName("height").item(0).getChildNodes().item(0).getNodeValue()));

            } catch (IOException ex) {
                Logger.getLogger(DesignPanelSelectArea.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    /**
     * Metoda, ktora vlozi obrazok do image panela.
     */
    private void insertImage() {
        imagePanel.removeAll();
        JScrollPane sp = new JScrollPane();
        sp.setBorder(null);
        if (imagePanel.getWidth() < imageComponent.getWidth() && imagePanel.getHeight() < imageComponent.getHeight()) {
            sp.setSize(imagePanel.getWidth(), imagePanel.getHeight());
        } else if (imagePanel.getWidth() < imageComponent.getWidth() && imagePanel.getHeight() > imageComponent.getHeight()) {
            sp.setSize(imagePanel.getWidth(), imagePanel.getHeight());
        } else if (imagePanel.getWidth() > imageComponent.getWidth() && imagePanel.getHeight() < imageComponent.getHeight()) {
            sp.setLocation((imagePanel.getWidth() - imageComponent.getWidth()) / 2, 0);
            sp.setSize(imageComponent.getWidth(), imagePanel.getHeight());
        } else if (imagePanel.getWidth() > imageComponent.getWidth() && imagePanel.getHeight() > imageComponent.getHeight()) {
            sp.setLocation((imagePanel.getWidth() - imageComponent.getWidth()) / 2, (imagePanel.getHeight() - imageComponent.getHeight()) / 2);
            sp.setSize(imageComponent.getWidth(), imageComponent.getHeight());
        }
        sp.setViewportView(imageComponent);
        imagePanel.add(sp);
    }

    /**
     * Metoda pozbiera informacie z formulara a vytvori info(definujuce otazku) o otazke mbimage select area. Udaje su tiez kontrolovane.
     * Skontroluje ci su udaje v poriadku. 
     * Pouziva sa pri generovani vysledneho xml suboru.
     * @return vrati info o otazke
     * @throws ValidationException ak otazka nesplna kriteria
     */
    public SelectAreaDefinition getSelectAreaDefinition() throws ValidationException {
        if (questionNameTextField.getText().isEmpty()) {
            throw new ValidationException("Nazov otazky musi byt definovany!");
        }
        if (imageComponent == null) {
            throw new ValidationException("Nie je definovany obrazok");
        }
        Rectangle2D.Double rect = imageComponent.getSelection();
        if (rect == null) {
            throw new ValidationException("Nie je definovane ziadne oznacenie");
        }
        return new SelectAreaDefinition(questionNameTextField.getText(), textOtazkyTextArea.getText(), imageComponent.getImage(), imageComponent.getImageName(), (int) rect.getX(), (int) rect.getY(), (int) rect.getWidth(), (int) rect.getHeight());

    }

    /**
     * Metoda pozbiera informacie z formulara a vytvori info(definujuce otazku) o otazke mbimage select area. Udaje nie su kontrolovane.
     * Pouziva sa pri nahlaiadnuti xml v programe(je to len informativne xml nie je potrebna kontrola). 
     * @return info o otazke
     */
    public SelectAreaDefinition getSelectAreaDef() {
        if (imageComponent != null) {
            Rectangle2D.Double rect = imageComponent.getSelection();
            if (imageComponent != null) {
                return new SelectAreaDefinition(questionNameTextField.getText(), textOtazkyTextArea.getText(), imageComponent.getImage(), imageComponent.getImageName(), (int) rect.getX(), (int) rect.getY(), (int) rect.getWidth(), (int) rect.getHeight());
            } else {
                return new SelectAreaDefinition(questionNameTextField.getText(), textOtazkyTextArea.getText(), imageComponent.getImage(), imageComponent.getImageName(), 0, 0, 0, 0);
            }
        } else {
            return new SelectAreaDefinition(questionNameTextField.getText(), textOtazkyTextArea.getText(), null, "", 0, 0, 0, 0);
        }

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        textOtazkyTextArea = new javax.swing.JTextArea();
        questionNameTextField = new javax.swing.JTextField();
        imageSettingsPanel = new javax.swing.JPanel();
        uploadButton = new javax.swing.JButton();
        resizeButton = new javax.swing.JButton();
        zoomOutButton = new javax.swing.JButton();
        zoomInButton = new javax.swing.JButton();
        jTextField1 = new javax.swing.JTextField();
        imagePanelBorder = new javax.swing.JPanel();
        imagePanel = new javax.swing.JPanel();

        setBackground(new java.awt.Color(255, 255, 255));

        jLabel1.setText("Text otázky :");

        jLabel2.setText("Názov otázky :");

        textOtazkyTextArea.setColumns(20);
        textOtazkyTextArea.setFont(new java.awt.Font("Tahoma", 0, 11)); // NOI18N
        textOtazkyTextArea.setRows(5);
        jScrollPane3.setViewportView(textOtazkyTextArea);

        imageSettingsPanel.setBackground(new java.awt.Color(255, 255, 255));
        imageSettingsPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Nastavenie obrázka"));
        imageSettingsPanel.setPreferredSize(new java.awt.Dimension(272, 107));

        uploadButton.setText("Upload");
        uploadButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                uploadButtonActionPerformed(evt);
            }
        });

        resizeButton.setText("Resize");
        resizeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resizeButtonActionPerformed(evt);
            }
        });

        zoomOutButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/img/magnifier-zoom-out.png"))); // NOI18N
        zoomOutButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zoomOutButtonActionPerformed(evt);
            }
        });

        zoomInButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/img/magnifier-zoom-in.png"))); // NOI18N
        zoomInButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zoomInButtonActionPerformed(evt);
            }
        });

        jTextField1.setEditable(false);

        javax.swing.GroupLayout imageSettingsPanelLayout = new javax.swing.GroupLayout(imageSettingsPanel);
        imageSettingsPanel.setLayout(imageSettingsPanelLayout);
        imageSettingsPanelLayout.setHorizontalGroup(
            imageSettingsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(imageSettingsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(imageSettingsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(imageSettingsPanelLayout.createSequentialGroup()
                        .addComponent(uploadButton)
                        .addGap(18, 18, 18)
                        .addComponent(zoomOutButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jTextField1, javax.swing.GroupLayout.DEFAULT_SIZE, 47, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(zoomInButton))
                    .addComponent(resizeButton))
                .addContainerGap())
        );
        imageSettingsPanelLayout.setVerticalGroup(
            imageSettingsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(imageSettingsPanelLayout.createSequentialGroup()
                .addGap(11, 11, 11)
                .addGroup(imageSettingsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(uploadButton)
                    .addComponent(zoomOutButton)
                    .addComponent(jTextField1, javax.swing.GroupLayout.PREFERRED_SIZE, 24, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(zoomInButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(resizeButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        imagePanelBorder.setBorder(javax.swing.BorderFactory.createTitledBorder(""));

        imagePanel.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                imagePanelComponentResized(evt);
            }
        });
        imagePanel.addContainerListener(new java.awt.event.ContainerAdapter() {
            public void componentAdded(java.awt.event.ContainerEvent evt) {
                imagePanelComponentAdded(evt);
            }
        });

        javax.swing.GroupLayout imagePanelLayout = new javax.swing.GroupLayout(imagePanel);
        imagePanel.setLayout(imagePanelLayout);
        imagePanelLayout.setHorizontalGroup(
            imagePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 666, Short.MAX_VALUE)
        );
        imagePanelLayout.setVerticalGroup(
            imagePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 377, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout imagePanelBorderLayout = new javax.swing.GroupLayout(imagePanelBorder);
        imagePanelBorder.setLayout(imagePanelBorderLayout);
        imagePanelBorderLayout.setHorizontalGroup(
            imagePanelBorderLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(imagePanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        imagePanelBorderLayout.setVerticalGroup(
            imagePanelBorderLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(imagePanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(imagePanelBorder, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jScrollPane3)
                            .addComponent(jLabel2)
                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                                .addComponent(jLabel1)
                                .addGap(316, 316, 316))
                            .addComponent(questionNameTextField))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(imageSettingsPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel2)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(questionNameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 34, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(imageSettingsPanel, javax.swing.GroupLayout.PREFERRED_SIZE, 100, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(imagePanelBorder, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void uploadButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_uploadButtonActionPerformed
        JFileChooser chooser = new JFileChooser();
        FileNameExtensionFilter filter = new FileNameExtensionFilter("Images", "jpeg", "jpg", "JPG", "JPEG", "gif", "png");
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            try {
                BufferedImage image = ImageIO.read(chooser.getSelectedFile());
                imagePanel.removeAll();
                imageComponent = new ImageComponentSelectArea(image, chooser.getSelectedFile().getName());
                imageComponent.setSize(image.getWidth(), image.getHeight());
                insertImage();
                jTextField1.setText("100");
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
    }//GEN-LAST:event_uploadButtonActionPerformed

    private void imagePanelComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_imagePanelComponentResized
        if (imageComponent != null) {
            insertImage();
        }
    }//GEN-LAST:event_imagePanelComponentResized

    private void imagePanelComponentAdded(java.awt.event.ContainerEvent evt) {//GEN-FIRST:event_imagePanelComponentAdded
        invalidate();
        updateUI();
        repaint();
    }//GEN-LAST:event_imagePanelComponentAdded

    private void zoomOutButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_zoomOutButtonActionPerformed
        int zoom = Integer.parseInt(jTextField1.getText());
        if (zoom != 10) {
            jTextField1.setText(String.valueOf(zoom - 10));
            imageComponent.zoom(((zoom) / 100.0), -0.1);
            imagePanel.removeAll();
            insertImage();
        }
    }//GEN-LAST:event_zoomOutButtonActionPerformed

    private void zoomInButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_zoomInButtonActionPerformed
        int zoom = Integer.parseInt(jTextField1.getText());
        if (zoom != 400) {
            jTextField1.setText(String.valueOf(zoom + 10));
            imageComponent.zoom((zoom) / 100.0, 0.1);
            imagePanel.removeAll();
            insertImage();
        }
    }//GEN-LAST:event_zoomInButtonActionPerformed

private void resizeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_resizeButtonActionPerformed
    if (imageComponent != null) {
        ResizeDialog dialog = new ResizeDialog(null, true, imageComponent);
        dialog.setVisible(true);
        imageComponent.imageResize(dialog.getValue());
        imagePanel.removeAll();
        insertImage();
        jTextField1.setText("100");
        repaint();
    } else {
        JOptionPane.showMessageDialog(this, "Nie je k dispozicii ziadny obrazok", "Zmena rozmerov", JOptionPane.INFORMATION_MESSAGE);
    }
}//GEN-LAST:event_resizeButtonActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel imagePanel;
    private javax.swing.JPanel imagePanelBorder;
    private javax.swing.JPanel imageSettingsPanel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JTextField questionNameTextField;
    private javax.swing.JButton resizeButton;
    private javax.swing.JTextArea textOtazkyTextArea;
    private javax.swing.JButton uploadButton;
    private javax.swing.JButton zoomInButton;
    private javax.swing.JButton zoomOutButton;
    // End of variables declaration//GEN-END:variables
}
