package it4kt.image.utils;

import it4kt.image.graphic.panels.MBimagePanel;
import interfaces.IModule;
import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JPanel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.SchemaFactory;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * Hlavna trieda pre otazku mbimage. 
 * Implementuje rozhranie IModule
 * @author marek
 */
public class Image implements IModule {

    /**
     * Vrati podporovane koncovky suborov.
     * @return 
     */
    public String[] getExtensions() {
        return new String[]{};
    }

    /**
     * Skontroluje ci subor je mozne spracovat. 
     * Ak je koncovka xml porovna sa ci xml vyhovuje scheme pre otazku mbimage
     * @param url
     * @return 
     */
    public boolean isFileAcceptable(String url) {
        try {
            getDocumentFromFile(url);
            return true;
        } catch (SAXException ex) {
            return false;
        }
    }

    /**
     * Ziskanie Documentu z xml suboru.
     * Ak je subor s koncovkou jmx musi sa najprv vykonat xslt transformacia, ktora transformuje jmx do pozadovaneho xml suboru.
     * Potom sa skontroluje ci vyhovuje scheme a xml sa transformuje do dokumentu.
     * Ak je xml potom sa hned kontroluje ci vyhovuje scheme a transformuje sa na dokument.
     * @param url
     * @return
     * @throws SAXException 
     */
    public Document getDocumentFromFile(String url) throws SAXException {
        String extension = url.substring(url.lastIndexOf(".") + 1, url.length());
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            factory.setNamespaceAware(true);
            SchemaFactory schemaFactory =
                    SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");

            factory.setSchema(schemaFactory.newSchema(
                    new Source[]{new StreamSource(getClass().getResourceAsStream("/xsd/mbimage.xsd"))}));
            DocumentBuilder builder = factory.newDocumentBuilder();
            builder.setErrorHandler(new SimpleErrorHandler());
            return builder.parse(new File(url));
        } catch (IOException ex) {
            Logger.getLogger(Image.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ParserConfigurationException ex) {
            System.out.println("parser exception");
            Logger.getLogger(Image.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    /**
     * vrati kod otazky
     * @return 
     */
    public String getCode() {
        return "mbimage";
    }

    /**
     * Vrati nazov otazky
     * @return 
     */
    public String getName() {
        return "Image question";
    }

    /**
     * Vrati ikonu otazky
     * @return 
     */
    public javax.swing.ImageIcon getImage() {
        return new javax.swing.ImageIcon(getClass().getResource("/img/icon.gif"));
    }

    /**
     * Vytvori novu alebo otvori staru otazku.
     * @param document
     * @return 
     */
    public JPanel execute(Document document) {
        return new MBimagePanel(document);
    }
}
