package it4kt.image.utils;

import it4kt.image.data.SelectAreaDefinition;
import it4kt.image.data.PickDefinition;
import java.io.StringWriter;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Trieda, ktora sa stara o vytvaranie xml
 * @author marek
 */
public class XML {

    private String url;

    /**
     * Vrati DOM strukturu otazky
     * @param SelectAreaDefinition
     * @return 
     */
    public DOMSource generateXML(SelectAreaDefinition SelectAreaDefinition) {
        try {
            SelectAreaDefinition info = (SelectAreaDefinition) SelectAreaDefinition;
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = documentBuilder.newDocument();

            Element question = document.createElementNS("http://xml.moodle.org/schema/it4kt_image", "question");
            document.appendChild(question);

            Element name = document.createElement("name");
            name.appendChild(document.createTextNode(info.getName()));
            question.appendChild(name);

            Element questiontext = document.createElement("questiontext");
            questiontext.appendChild(document.createTextNode(info.getQuestiontext()));
            question.appendChild(questiontext);

            Element imagetype = document.createElement("imagetype");
            imagetype.appendChild(document.createTextNode("selectarea"));
            question.appendChild(imagetype);

            Element image = document.createElement("image");
            image.setAttribute("name", info.getImageName());
            image.setAttribute("encoding", "base64");
            image.appendChild(document.createTextNode(info.getImage()));
            question.appendChild(image);

            Element selectarea = document.createElement("selectarea");
            Element centerpoint = document.createElement("centerpoint");
            Element x = document.createElement("x");
            x.appendChild(document.createTextNode(String.valueOf(info.getX())));
            centerpoint.appendChild(x);
            Element y = document.createElement("y");
            y.appendChild(document.createTextNode(String.valueOf(info.getY())));
            centerpoint.appendChild(y);
            selectarea.appendChild(centerpoint);

            Element dimensions = document.createElement("dimensions");
            Element width = document.createElement("width");
            width.appendChild(document.createTextNode(String.valueOf(info.getWidth())));
            dimensions.appendChild(width);
            Element height = document.createElement("height");
            height.appendChild(document.createTextNode(String.valueOf(info.getHeight())));
            dimensions.appendChild(height);
            selectarea.appendChild(dimensions);
            question.appendChild(selectarea);
            DOMSource source = new DOMSource(document);
            return source;
        } catch (ParserConfigurationException ex) {
            Logger.getLogger(XML.class.getName()).log(Level.SEVERE, null, ex);
            return null;
        }

    }

    /**
     * Vrati DOM strukturu otazky
     * @param pickDefinition
     * @return 
     */
    public DOMSource generateXML(PickDefinition pickDefinition) {
        try {
            PickDefinition info = (PickDefinition) pickDefinition;
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = documentBuilder.newDocument();

            Element question = document.createElementNS("http://xml.moodle.org/schema/it4kt_image", "question");
            document.appendChild(question);

            Element name = document.createElement("name");
            name.appendChild(document.createTextNode(info.getName()));
            question.appendChild(name);

            Element questiontext = document.createElement("questiontext");
            questiontext.appendChild(document.createTextNode(info.getQuestiontext()));
            question.appendChild(questiontext);

            Element imagetype = document.createElement("imagetype");
            imagetype.appendChild(document.createTextNode("pick"));
            question.appendChild(imagetype);

            Element image = document.createElement("image");
            image.setAttribute("name", info.getImageName());
            image.setAttribute("encoding", "base64");
            image.appendChild(document.createTextNode(info.getImage()));
            question.appendChild(image);

            Element pick = document.createElement("pick");
            Element singleselect = document.createElement("singleselect");
            singleselect.appendChild(document.createTextNode(String.valueOf(info.getSingleselect())));
            pick.appendChild(singleselect);

            Element answers = document.createElement("answers");
            for (int i = 0; i < info.getAnswers().size(); i++) {
                Element answer = document.createElement("answer");
                Element shape = document.createElement("shape");
                shape.appendChild(document.createTextNode(info.getAnswers().get(i).getType()));
                answer.appendChild(shape);
                Element coords = document.createElement("coords");
                for (int j = 0; j < info.getAnswers().get(i).getCoords().size(); j++) {
                    Element p = document.createElement("p");
                    p.appendChild(document.createTextNode(info.getAnswers().get(i).getCoords().get(j).toString()));
                    coords.appendChild(p);
                }
                answer.appendChild(coords);
                Element description = document.createElement("description");
                description.appendChild(document.createTextNode(info.getAnswers().get(i).getDescription()));
                answer.appendChild(description);
                Element code = document.createElement("code");
                code.appendChild(document.createTextNode(String.valueOf(info.getAnswers().get(i).getCode())));
                answer.appendChild(code);
                Element correct = document.createElement("correct");
                correct.appendChild(document.createTextNode(String.valueOf(info.getAnswers().get(i).getCorrect())));
                answer.appendChild(correct);
                answers.appendChild(answer);
            }

            pick.appendChild(answers);
            question.appendChild(pick);
            DOMSource source = new DOMSource(document);
            return source;
        } catch (ParserConfigurationException ex) {
            Logger.getLogger(XML.class.getName()).log(Level.SEVERE, null, ex);
            return null;
        }

    }

    /**
     * Vrati textovu reprezentaciu DOM struktury
     * @param source
     * @return 
     */
    public String generateStringFromXML(DOMSource source) {
        try {
            StringWriter stw = new StringWriter();
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            transformerFactory.setAttribute("indent-number", 2);
            Transformer serializer = transformerFactory.newTransformer();
            serializer.setOutputProperty(OutputKeys.INDENT, "yes");
            serializer.transform(source, new StreamResult(stw));
            return stw.toString();
        } catch (TransformerException ex) {
            Logger.getLogger(XML.class.getName()).log(Level.SEVERE, null, ex);
            return "";
        }
    }

    /**
     * @return the url
     */
    public String getUrl() {
        return url;
    }

    /**
     * @param url the url to set
     */
    public void setUrl(String url) {
        this.url = url;
    }
}
