package it4kt.match.graphic;

import it4kt.match.graphic.dialogs.GetAnswer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.ListModel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.event.MouseInputAdapter;
import it4kt.match.utils.Answer;

/**
 *
 * @author marek
 */
public class ReorderableList extends JList {

    private final JPanel p = new JPanel(new BorderLayout());
    private final JPanel p1 = new JPanel(new BorderLayout());
    private final JLabel label = new JLabel("", JLabel.LEFT);
    private final JLabel label2 = new JLabel("", JLabel.LEFT);
    private final JLabel icon = new JLabel((Icon) null, JLabel.CENTER);

    public ReorderableList(ListModel model) {
        super(model);
        setFixedCellHeight(200);
        setFixedCellWidth(300);

        label.setOpaque(true);
        //label.setForeground(getForeground());
        //label.setBackground(Color.YELLOW);

        label2.setOpaque(true);
        //label2.setForeground(getForeground());

        p.setOpaque(false);
        p.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));

        p1.setOpaque(false);
        p1.setBorder(BorderFactory.createEtchedBorder());
        p1.setSize(p.getSize());
        p.add(p1);
        setCellRenderer(new ListCellRenderer() {

            @Override
            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                Answer item = (Answer) getModel().getElementAt(index);
                label.setText(item.getQuestion());

                label.setBorder(new EtchedBorder());
                label.setFont(new Font("Arial", 1, 14));
                label.setVerticalAlignment(SwingConstants.CENTER);
                label.setHorizontalAlignment(JLabel.LEADING);
                label.setForeground(isSelected ? Color.BLACK : new Color(107, 128, 139));
                label.setBackground(isSelected ? new Color(252, 239, 161) : new Color(237, 237, 237));

                if (item.getImage() == null) {
                    label2.setText(item.getAnswertext());

                    label2.setBorder(new EtchedBorder());
                    label2.setFont(new Font("Arial", 1, 12));
                    label2.setVerticalAlignment(SwingConstants.CENTER);
                    label2.setHorizontalAlignment(JLabel.LEADING);
                    label2.setForeground(isSelected ? Color.BLACK : new Color(107, 128, 139));
                    label2.setBackground(isSelected ? new Color(252, 239, 161) : new Color(237, 237, 237));
                    p1.remove(icon);
                    p1.add(label2, BorderLayout.CENTER);
                    p1.add(label, BorderLayout.BEFORE_FIRST_LINE);
                } else {
                    icon.setIcon(new ImageIcon(item.getImage()));

                    p1.remove(label2);
                    p1.add(icon, BorderLayout.CENTER);
                    p1.add(label, BorderLayout.BEFORE_FIRST_LINE);
                }
                return p;
            }
        });
        RubberBandingListener rbl = new RubberBandingListener();
        addMouseMotionListener(rbl);
        addMouseListener(rbl);

    }

    class RubberBandingListener extends MouseInputAdapter {

        private Point srcPoint = null;
        private final Polygon polygon = new Polygon();

        @Override
        public void mouseDragged(MouseEvent e) {
            JList list = (JList) e.getSource();
            if (list.getDragEnabled()) {
                return;
            }
            if (srcPoint == null) {
                srcPoint = e.getPoint();
            }
            Point destPoint = e.getPoint();
            polygon.reset();
            polygon.addPoint(srcPoint.x, srcPoint.y);
            polygon.addPoint(destPoint.x, srcPoint.y);
            polygon.addPoint(destPoint.x, destPoint.y);
            polygon.addPoint(srcPoint.x, destPoint.y);
            setSelectedIndices(getIntersectsIcons(polygon));
            repaint();
        }

        @Override
        public void mouseReleased(MouseEvent e) {
            setFocusable(true);
            if (srcPoint == null || !getDragEnabled()) {
                JList list = (JList) e.getSource();
                Component glassPane = list.getRootPane().getGlassPane();
                glassPane.setVisible(false);
            }
            srcPoint = null;
            setDragEnabled(getSelectedIndices().length > 0);
            repaint();
        }

        @Override
        public void mousePressed(MouseEvent e) {
            int index = locationToIndex(e.getPoint());
            Rectangle rect = getCellBounds(index, index);
            if (!rect.contains(e.getPoint())) {
                JList list = (JList) e.getSource();
                Component glassPane = list.getRootPane().getGlassPane();
                glassPane.setVisible(false);

                clearSelection();
                getSelectionModel().setAnchorSelectionIndex(-1);
                getSelectionModel().setLeadSelectionIndex(-1);
                setFocusable(false);
                setDragEnabled(false);
            } else {
                setFocusable(true);
                if (getDragEnabled()) {
                    return;
                } else {
                    setSelectedIndex(index);
                }
            }
            repaint();
        }

        @Override
        public void mouseClicked(MouseEvent e) {
            JList list = (JList) e.getSource();
            if (e.getClickCount() == 1 & e.getButton() == 2) {
                if (list.getSelectedIndex() != -1) {
                    ((DefaultListModel) list.getModel()).remove(list.getSelectedIndex());
                }
            }
            if (e.getClickCount() == 2 && e.getButton() == 1) {
                int index = list.getSelectedIndex();
                if (index != -1) {
                    DefaultListModel dlm = (DefaultListModel) list.getModel();
                    Answer item = (Answer) dlm.getElementAt(index);
                    GetAnswer dialog = new GetAnswer(null, true, item);
                    dialog.setVisible(true);
                    item.setQuestion(dialog.getValue().getQuestion());
                    if (dialog.getValue().getImage() == null) {
                        item.setAnswertext(dialog.getValue().getAnswertext());
                        item.setImage(null);
                    } else {
                        item.setImage(dialog.getValue().getImage());
                        item.setImageName(dialog.getValue().getImageName());
                        item.setAnswertext("");
                    }
                }
            } else if (e.getClickCount() == 2 && e.getButton() == 3) {
                DefaultListModel dlm = (DefaultListModel) list.getModel();
                Answer item = new Answer(1);
                GetAnswer dialog = new GetAnswer(null, true, item);
                dialog.setVisible(true);
                item.setQuestion(dialog.getValue().getQuestion());
                if (dialog.getValue().getImage() == null) {
                    item.setAnswertext(dialog.getValue().getAnswertext());
                    item.setImage(null);
                } else {
                    item.setImage(dialog.getValue().getImage());
                    item.setImageName(dialog.getValue().getImageName());
                    item.setAnswertext("");
                }
                dlm.addElement(item);

            }
        }

        private int[] getIntersectsIcons(Shape p) {
            ListModel model = getModel();
            List<Integer> list = new ArrayList<Integer>(model.getSize());
            for (int i = 0; i < model.getSize(); i++) {
                Rectangle r = getCellBounds(i, i);
                if (p.intersects(r)) {
                    list.add(i);
                }
            }
            int[] il = new int[list.size()];
            for (int i = 0; i < list.size(); i++) {
                il[i] = list.get(i);
            }
            return il;
        }
    }
}
