package it4kt.match.graphic.panels;

import exceptions.ValidationException;
import it4kt.match.graphic.ListItemTransferHandler;
import it4kt.match.graphic.ReorderableList;
import it4kt.match.graphic.dialogs.GetAnswer;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.DefaultListModel;
import it4kt.match.utils.Answer;
import it4kt.match.utils.MatchDefinition;
import org.apache.commons.codec.binary.Base64;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Tento panel sa vyuziva na graficku tvorbu otazky (mborder).
 * @author marek
 */
public class DesignPanel extends javax.swing.JPanel {

    //list model - pouziva sa na ukladanie odpovedi na otazku
    DefaultListModel model = new DefaultListModel();

    /**
     * Creates new form DesignPanel
     * Konstruktor triedy DesignPanel. Vytvoria a nastavia sa vsetky preddefinovane hodnoty. 
     * Ak je zadany vstupny subor potom sa analyzuje a vytvori sa jeho graficka reprezentacia.
     * @param document document vytvoreny zo vstupneho xml 
     */
    public DesignPanel(Document document) {
        initComponents();
        //vytvorenie grafickej reprezentacie
        initQuestion(document);
        //nastavovanie list pre odpovede
        answersList.setModel(model);
        answersList.setTransferHandler(new ListItemTransferHandler());
        //pridanie podpory pre klavesy do listu - delete(odstrani odpoved), enter(otvori dialog pre pridanie textu odpovede)
        answersList.addKeyListener(new KeyListener() {

            public void keyTyped(KeyEvent e) {
            }

            public void keyPressed(KeyEvent e) {
            }

            public void keyReleased(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_DELETE) {
                    ((DefaultListModel) answersList.getModel()).remove(answersList.getSelectedIndex());
                } else if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    int index = answersList.getSelectedIndex();
                    if (index != -1) {
                        DefaultListModel dlm = (DefaultListModel) answersList.getModel();
                        Answer item = (Answer) dlm.getElementAt(index);
                        GetAnswer dialog = new GetAnswer(null, true, item);
                        dialog.setVisible(true);
                        item.setQuestion(dialog.getValue().getQuestion());
                        if (dialog.getValue().getImage() == null) {
                            item.setAnswertext(dialog.getValue().getAnswertext());
                            item.setImage(null);
                        } else {
                            item.setImage(dialog.getValue().getImage());
                            item.setImageName(dialog.getValue().getImageName());
                            item.setAnswertext("");
                        }
                    }
                }
            }
        });
    }

    /**
     * Metoda nastavuje hodnoty pre otazku mborder
     * Ak dokument existuje potom sa nastavia hodnoty otazky z tohto dokumentu.
     * Ak dokument neexistuje potom sa nastavia preddefinovane hodnoty.
     * @param document 
     */
    private void initQuestion(Document document) {
        if (document == null) {
            //defaultne prida tri prazdne odpovede
            model = new DefaultListModel();
            model.addElement(new Answer(1));
            model.addElement(new Answer(2));
            model.addElement(new Answer(3));
        } else {
            Element root = document.getDocumentElement();
            //nastavi sa meno otazky
            if (root.getElementsByTagName("name").item(0).getChildNodes().getLength() != 0) {
                questionNameTextField.setText(root.getElementsByTagName("name").item(0).getChildNodes().item(0).getNodeValue());
            }
            //nastavi sa text otazky
            if (root.getElementsByTagName("questiontext").item(0).getChildNodes().getLength() != 0) {
                questionTextTextField.setText(root.getElementsByTagName("questiontext").item(0).getChildNodes().item(0).getNodeValue());
            }
            //ziskaju a nastavia sa odpovede
            model = new DefaultListModel();
            NodeList nodelist = root.getElementsByTagName("answer");
            for (int i = 0; i < nodelist.getLength(); i++) {
                Element answerNode = (Element) nodelist.item(i);

                //defaultne hodnoty
                String answertext = "";
                String subquestion = "";
                BufferedImage image = null;
                String imageName = "";
                int code = -1;
                try {
                    if (answerNode.getElementsByTagName("answertext").item(0).getChildNodes().getLength() != 0) {
                        answertext = answerNode.getElementsByTagName("answertext").item(0).getChildNodes().item(0).getNodeValue();
                    }
                    if (answerNode.getElementsByTagName("subquestion").item(0).getChildNodes().getLength() != 0) {
                        subquestion = answerNode.getElementsByTagName("subquestion").item(0).getChildNodes().item(0).getNodeValue();
                    }
                    if (answerNode.getElementsByTagName("code").item(0).getChildNodes().getLength() != 0) {
                        code = Integer.parseInt(answerNode.getElementsByTagName("code").item(0).getChildNodes().item(0).getNodeValue());
                    }
                    model.addElement(new Answer(subquestion, answertext, code));
                } catch (NullPointerException e) {
                    if (answerNode.getElementsByTagName("answerimage").item(0).getChildNodes().getLength() != 0) {
                        byte[] imageBytes = Base64.decodeBase64(answerNode.getElementsByTagName("answerimage").item(0).getChildNodes().item(0).getNodeValue());
                        InputStream in = new ByteArrayInputStream(imageBytes);
                        try {
                            image = ImageIO.read(in);
                            int width = 96;
                            int height = 72;
                            int imagewidth = ((BufferedImage) image).getWidth();
                            int imageheight = ((BufferedImage) image).getHeight();
                            BufferedImage image_scale = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
                            Graphics2D g = image_scale.createGraphics();
                            AffineTransform at =
                                    AffineTransform.getScaleInstance((double) width / imagewidth,
                                    (double) height / imageheight);
                            g.drawRenderedImage((BufferedImage) image, at);
                            image = image_scale;
                        } catch (IOException ex) {
                            Logger.getLogger(DesignPanel.class.getName()).log(Level.SEVERE, null, ex);
                        }
                        imageName = answerNode.getElementsByTagName("answerimage").item(0).getAttributes().getNamedItem("name").getChildNodes().item(0).getNodeValue();
                    }
                    if (answerNode.getElementsByTagName("subquestion").item(0).getChildNodes().getLength() != 0) {
                        subquestion = answerNode.getElementsByTagName("subquestion").item(0).getChildNodes().item(0).getNodeValue();
                    }
                    if (answerNode.getElementsByTagName("code").item(0).getChildNodes().getLength() != 0) {
                        code = Integer.parseInt(answerNode.getElementsByTagName("code").item(0).getChildNodes().item(0).getNodeValue());
                    }
                    model.addElement(new Answer(subquestion, image, imageName, code));
                }
            }
        }
    }

    /**
     * Metoda pozbiera informacie z formulara a vytvori info(definujuce otazku) o otazke mborder. Udaje su tiez kontrolovane.
     * Najprv ziska odpovede a precisluje ich podla poradia. 
     * Skontroluje ci je zadany nazov otazky a pocet odpovedi je viac ako tri.
     * Pouziva sa pri generovani vysledneho xml suboru.
     * @return vrati info o otazke
     * @throws ValidationException ak otazka nesplna kriteria
     */
    public MatchDefinition getMatchDefWithValidation() throws ValidationException {
        List anss = new ArrayList();
        for (int i = 0; i < answersList.getModel().getSize(); i++) {
            Answer ans = (Answer) answersList.getModel().getElementAt(i);
            anss.add(ans);
        }
        if (questionNameTextField.getText().isEmpty()) {
            throw new ValidationException("Názov otázky musí byť definovaný!");
        }
        if (anss.size() < 3) {
            throw new ValidationException("Počet možnosti musí byť minimálne 3!");
        }
        return new MatchDefinition(questionNameTextField.getText(), questionTextTextField.getText(), anss);
    }

    /**
     * Metoda pozbiera informacie z formulara a vytvori info(definujuce otazku) o otazke mborder. Udaje nie su kontrolovane.
     * Pouziva sa pri nahlaiadnuti xml v programe(je to len informativne xml nie je potrebna kontrola). 
     * @return info o otazke
     */
    public MatchDefinition getMatchDefWithoutValidation() {
        List anss = new ArrayList();
        for (int i = 0; i < answersList.getModel().getSize(); i++) {
            Answer ans = (Answer) answersList.getModel().getElementAt(i);
            anss.add(ans);
        }
        return new MatchDefinition(questionNameTextField.getText(), questionTextTextField.getText(), anss);

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        questionTextLabel = new javax.swing.JLabel();
        questionTextTextField = new javax.swing.JTextField();
        answersListScrollPane = new javax.swing.JScrollPane();
        answersList = new ReorderableList(model);
        questionNameLabel = new javax.swing.JLabel();
        questionNameTextField = new javax.swing.JTextField();

        setBackground(new java.awt.Color(255, 255, 255));

        questionTextLabel.setText("Text otázky :");

        answersList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        answersList.setDropMode(javax.swing.DropMode.INSERT);
        answersList.setLayoutOrientation(javax.swing.JList.HORIZONTAL_WRAP);
        answersList.setVisibleRowCount(2);
        answersListScrollPane.setViewportView(answersList);

        questionNameLabel.setText("Názov otázky :");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addGroup(layout.createSequentialGroup()
                            .addContainerGap()
                            .addComponent(questionNameLabel))
                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                            .addContainerGap()
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addComponent(questionTextLabel)
                                .addComponent(questionTextTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 380, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addGroup(layout.createSequentialGroup()
                            .addContainerGap()
                            .addComponent(questionNameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 380, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(answersListScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 380, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(questionNameLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(questionNameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(questionTextLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(questionTextTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(answersListScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 250, Short.MAX_VALUE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JList answersList;
    private javax.swing.JScrollPane answersListScrollPane;
    private javax.swing.JLabel questionNameLabel;
    private javax.swing.JTextField questionNameTextField;
    private javax.swing.JLabel questionTextLabel;
    private javax.swing.JTextField questionTextTextField;
    // End of variables declaration//GEN-END:variables
}
