package it4kt.match.utils;

import it4kt.match.graphic.panels.MBmatchPanel;
import interfaces.IModule;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JPanel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.SchemaFactory;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Hlavna trieda pre otazku mborder. 
 * Implementuje rozhranie IModule
 * @author marek
 */
public class Match implements IModule {

    public String[] getExtensions() {
        return new String[]{"xml", "jmt"};
    }

    public boolean isFileAcceptable(String url) {
        String extension = url.substring(url.lastIndexOf(".") + 1, url.length());
        if ("jmt".equals(extension)) {
            return true;
        } else if ("xml".equals(extension)) {
            try {
                getDocumentFromFile(url);
                return true;
            } catch (SAXException ex) {
                return false;
            }
        }
        return false;
    }

    public Document getDocumentFromFile(String url) throws SAXException {
        String extension = url.substring(url.lastIndexOf(".") + 1, url.length());
        try {
            if (extension.equals("jmt")) {
                javax.xml.transform.Source xmlSource =
                        new javax.xml.transform.stream.StreamSource(new File(url));
                javax.xml.transform.Source xsltSource =
                        new javax.xml.transform.stream.StreamSource(getClass().getResourceAsStream("/xslt/mbmatch.xsl"));
                StreamResult result = new StreamResult(new StringWriter());

                // create an instance of TransformerFactory
                javax.xml.transform.TransformerFactory transFact =
                        javax.xml.transform.TransformerFactory.newInstance();

                javax.xml.transform.Transformer trans =
                        transFact.newTransformer(xsltSource);
                trans.setParameter("file_loc", url.substring(0, url.lastIndexOf("\\") + 1));
                trans.transform(xmlSource, result);
                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                factory.setValidating(false);
                factory.setNamespaceAware(true);
                SchemaFactory schemaFactory =
                        SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");

                factory.setSchema(schemaFactory.newSchema(
                        new Source[]{new StreamSource(getClass().getResourceAsStream("/xsd/mbmatch.xsd"))}));
                DocumentBuilder builder = factory.newDocumentBuilder();
                return builder.parse(new InputSource(new ByteArrayInputStream(result.getWriter().toString().getBytes())));
            }
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            factory.setNamespaceAware(true);
            SchemaFactory schemaFactory =
                    SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");

            factory.setSchema(schemaFactory.newSchema(
                    new Source[]{new StreamSource(getClass().getResourceAsStream("/xsd/mbmatch.xsd"))}));
            DocumentBuilder builder = factory.newDocumentBuilder();
            builder.setErrorHandler(new SimpleErrorHandler());
            return builder.parse(new File(url));
        } catch (TransformerException ex) {
            Logger.getLogger(Match.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(Match.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ParserConfigurationException ex) {
            System.out.println("parser exception");
            Logger.getLogger(Match.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    /**
     * vrati kod otazky
     * @return 
     */
    public String getCode() {
        return "it4kt_match";
    }

    /**
     * Vrati nazov otazky
     * @return 
     */
    public String getName() {
        return "Match question";
    }

    /**
     * Vrati ikonu otazky
     * @return 
     */
    public javax.swing.ImageIcon getImage() {
        return new javax.swing.ImageIcon(getClass().getResource("/img/icon.gif"));
    }

    /**
     * Vytvori novu alebo otvori staru otazku.
     * @param document
     * @return 
     */
    public JPanel execute(Document document) {
        return new MBmatchPanel(document);
    }
}
