package it4kt.match.utils;

import java.io.StringWriter;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Trieda na generovanie xml
 * @author marek
 */
public class XMLGeneration {

    /**
     * Vrati DOM strukturu otazky
     * @param OrderDefinition
     * @return 
     */
    public static DOMSource generateXML(MatchDefinition info) {
        try {
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = documentBuilder.newDocument();

            //question element
            Element question = document.createElementNS("http://xml.moodle.org/schema/it4kt_match", "question");
            document.appendChild(question);

            //name element
            Element name = document.createElement("name");
            name.appendChild(document.createTextNode(info.getName()));
            question.appendChild(name);

            //question text element
            Element questiontext = document.createElement("questiontext");
            questiontext.appendChild(document.createTextNode(info.getQuestiontext()));
            question.appendChild(questiontext);

            //answers element
            Element answers = document.createElement("answers");
            for (int i = 0; i < info.getAnswers().size(); i++) {
                Element answer = document.createElement("answer");
                //answertext element
                Element subquestion = document.createElement("subquestion");
                Answer ans = (Answer) info.getAnswers().get(i);
                subquestion.appendChild(document.createTextNode(ans.getQuestion()));
                answer.appendChild(subquestion);
                //position element
                if (ans.getImage() == null) {
                    Element answertext = document.createElement("answertext");
                    answertext.appendChild(document.createTextNode(ans.getAnswertext()));
                    answer.appendChild(answertext);
                } else {
                    Element answerimage = document.createElement("answerimage");
                    answerimage.setAttribute("name", ans.getImageName());
                    answerimage.setAttribute("encoding", "base64");
                    answerimage.appendChild(document.createTextNode(ans.getImageBin()));
                    answer.appendChild(answerimage);
                }
                //code element
                Element code = document.createElement("code");
                code.appendChild(document.createTextNode(String.valueOf(((Answer) info.getAnswers().get(i)).getCode())));
                answer.appendChild(code);
                answers.appendChild(answer);
            }
            question.appendChild(answers);

            DOMSource source = new DOMSource(document);
            return source;
        } catch (ParserConfigurationException ex) {
            Logger.getLogger(Match.class.getName()).log(Level.SEVERE, null, ex);
            return null;
        }
    }

    /**
     * Vrati textovu reprezentaciu DOM struktury
     * @param source
     * @return 
     */
    public static String generateStringFromXML(DOMSource source) {
        try {
            StringWriter stw = new StringWriter();
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            transformerFactory.setAttribute("indent-number", 2);
            Transformer serializer = transformerFactory.newTransformer();
            serializer.setOutputProperty(OutputKeys.INDENT, "yes");
            serializer.transform(source, new StreamResult(stw));
            return stw.toString();
        } catch (TransformerException ex) {
            Logger.getLogger(XMLGeneration.class.getName()).log(Level.SEVERE, null, ex);
            return "";
        }
    }
}
