package it4kt.order.graphic;

import it4kt.order.graphic.dialogs.GetAnswer;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.event.MouseInputAdapter;
import it4kt.order.data.Answer;

/**
 * Trieda, ktora rozsiruje jList a robi ho usporiadatelnym (mysli sa tym usporiadanie vyuzitim drag and drop)
 * @author marek
 */
public class ReorderableList extends JList {

    private final JPanel p = new JPanel(new BorderLayout());
    private final JLabel label = new JLabel("", JLabel.LEFT);

    /**
     * Konstruktor triedy. Vytvara zobrazenie pre kazdu odpoved. Najprv sa nastavi fixna velkost stlpca. 
     * Potom sa vytvori element panel a v nom element label. 
     * Taktiez sa v nom definuje listCellRenderer, ktory vrati zobrazenie pre kazdy odpoved(stlpec).
     * @param model 
     */
    public ReorderableList(ListModel model) {
        super(model);
        setFixedCellHeight(50);

        label.setOpaque(true);
        label.setForeground(getForeground());
        label.setBackground(Color.GRAY);

        p.setOpaque(false);
        p.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));
        p.add(label, BorderLayout.CENTER);

        setCellRenderer(new ListCellRenderer() {

            @Override
            public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                Answer item = (Answer) getModel().getElementAt(index);
                label.setText(item.getAnswer());

                label.setBorder(new EtchedBorder());
                label.setFont(new Font("Arial", 1, 25));
                label.setVerticalAlignment(SwingConstants.CENTER);
                label.setHorizontalAlignment(JLabel.LEADING);
                label.setForeground(isSelected ? Color.BLACK : new Color(107, 128, 139));
                label.setBackground(isSelected ? new Color(252, 239, 161) : new Color(237, 237, 237));

                return p;
            }
        });
        RubberBandingListener rbl = new RubberBandingListener();
        addMouseMotionListener(rbl);
        addMouseListener(rbl);

    }

    /**
     * Definuje spravanie sa usporiadatelneho listu ak sa stlaci mys.
     */
    class RubberBandingListener extends MouseInputAdapter {

        private Point srcPoint = null;
        private final Polygon polygon = new Polygon();

        @Override
        public void mouseDragged(MouseEvent e) {
            JList list = (JList) e.getSource();
            if (list.getDragEnabled()) {
                return;
            }
            if (srcPoint == null) {
                srcPoint = e.getPoint();
            }
            Point destPoint = e.getPoint();
            polygon.reset();
            polygon.addPoint(srcPoint.x, srcPoint.y);
            polygon.addPoint(destPoint.x, srcPoint.y);
            polygon.addPoint(destPoint.x, destPoint.y);
            polygon.addPoint(srcPoint.x, destPoint.y);
            setSelectedIndices(getIntersectsIcons(polygon));
            repaint();
        }

        @Override
        public void mouseReleased(MouseEvent e) {
            setFocusable(true);
            if (srcPoint == null || !getDragEnabled()) {
                JList list = (JList) e.getSource();
                Component glassPane = list.getRootPane().getGlassPane();
                glassPane.setVisible(false);
            }
            srcPoint = null;
            setDragEnabled(getSelectedIndices().length > 0);
            repaint();
        }

        @Override
        public void mousePressed(MouseEvent e) {
            int index = locationToIndex(e.getPoint());
            Rectangle rect = getCellBounds(index, index);
            if (!rect.contains(e.getPoint())) {
                JList list = (JList) e.getSource();
                Component glassPane = list.getRootPane().getGlassPane();
                glassPane.setVisible(false);

                clearSelection();
                getSelectionModel().setAnchorSelectionIndex(-1);
                getSelectionModel().setLeadSelectionIndex(-1);
                setFocusable(false);
                setDragEnabled(false);
            } else {
                setFocusable(true);
                if (getDragEnabled()) {
                    return;
                } else {
                    setSelectedIndex(index);
                }
            }
            repaint();
        }

        @Override
        public void mouseClicked(MouseEvent e) {
            JList list = (JList) e.getSource();
            if (e.getClickCount() == 1 & e.getButton() == 2) {
                if (list.getSelectedIndex() != -1) {
                    ((DefaultListModel) list.getModel()).remove(list.getSelectedIndex());
                }
            }
            if (e.getClickCount() == 2 && e.getButton() == 1) {
                int index = list.getSelectedIndex();
                if (index != -1) {
                    DefaultListModel dlm = (DefaultListModel) list.getModel();
                    Answer item = (Answer) dlm.getElementAt(index);
                    GetAnswer dialog = new GetAnswer(null, true, item.getAnswer());
                    dialog.setVisible(true);
                    item.setAnswer(dialog.getValue());
                }
            } else if (e.getClickCount() == 2 && e.getButton() == 3) {
                DefaultListModel dlm = (DefaultListModel) list.getModel();
                Answer item = new Answer(list.getComponentCount());
                GetAnswer dialog = new GetAnswer(null, true, item.getAnswer());
                dialog.setVisible(true);
                item.setAnswer(dialog.getValue());
                dlm.addElement(item);

            }
        }

        private int[] getIntersectsIcons(Shape p) {
            ListModel model = getModel();
            List<Integer> list = new ArrayList<Integer>(model.getSize());
            for (int i = 0; i < model.getSize(); i++) {
                Rectangle r = getCellBounds(i, i);
                if (p.intersects(r)) {
                    list.add(i);
                }
            }
            int[] il = new int[list.size()];
            for (int i = 0; i < list.size(); i++) {
                il[i] = list.get(i);
            }
            return il;
        }
    }
}
