package it4kt.order.graphic.panels;

import exceptions.ValidationException;
import it4kt.order.graphic.ListItemTransferHandler;
import it4kt.order.graphic.ReorderableList;
import it4kt.order.graphic.dialogs.GetAnswer;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.List;
import javax.swing.DefaultListModel;
import it4kt.order.data.Answer;
import it4kt.order.data.OrderDefinition;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Tento panel sa vyuziva na graficku tvorbu otazky (mborder).
 * @author marek
 */
public class DesignPanel extends javax.swing.JPanel {

    //list model - pouziva sa na ukladanie odpovedi na otazku
    DefaultListModel model = new DefaultListModel();

    /**
     * Creates new form DesignPanel
     * Konstruktor triedy DesignPanel. Vytvoria a nastavia sa vsetky preddefinovane hodnoty. 
     * Ak je zadany vstupny subor potom sa analyzuje a vytvori sa jeho graficka reprezentacia.
     * @param document document vytvoreny zo vstupneho xml 
     */
    public DesignPanel(Document document) {
        initComponents();
        //vytvorenie grafickej reprezentacie
        initQuestion(document);
        //nastavovanie list pre odpovede
        answersList.setModel(model);
        answersList.setTransferHandler(new ListItemTransferHandler());
        //pridanie podpory pre klavesy do listu - delete(odstrani odpoved), enter(otvori dialog pre pridanie textu odpovede)
        answersList.addKeyListener(new KeyListener() {

            public void keyTyped(KeyEvent e) {
            }

            public void keyPressed(KeyEvent e) {
            }

            public void keyReleased(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_DELETE) {
                    ((DefaultListModel) answersList.getModel()).remove(answersList.getSelectedIndex());
                } else if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    int index = answersList.getSelectedIndex();
                    if (index != -1) {
                        DefaultListModel dlm = (DefaultListModel) answersList.getModel();
                        Answer item = (Answer) dlm.getElementAt(index);
                        GetAnswer dialog = new GetAnswer(null, true, item.getAnswer());
                        dialog.setVisible(true);
                        item.setAnswer(dialog.getValue());
                    }
                }
            }
        });
    }

    /**
     * Metoda nastavuje hodnoty pre otazku mborder
     * Ak dokument existuje potom sa nastavia hodnoty otazky z tohto dokumentu.
     * Ak dokument neexistuje potom sa nastavia preddefinovane hodnoty.
     * @param document 
     */
    private void initQuestion(Document document) {
        if (document == null) {
            //defaultne prida tri prazdne odpovede
            model = new DefaultListModel();
            model.addElement(new Answer(0));
            model.addElement(new Answer(1));
            model.addElement(new Answer(2));
        } else {
            Element root = document.getDocumentElement();
            //nastavi sa meno otazky
            if (root.getElementsByTagName("name").item(0).getChildNodes().getLength() != 0) {
                questionNameTextField.setText(root.getElementsByTagName("name").item(0).getChildNodes().item(0).getNodeValue());
            }
            //nastavi sa text otazky
            if (root.getElementsByTagName("questiontext").item(0).getChildNodes().getLength() != 0) {
                questionTextTextArea.setText(root.getElementsByTagName("questiontext").item(0).getChildNodes().item(0).getNodeValue());
            }
            //ziskaju a nastavia sa odpovede
            model = new DefaultListModel();
            NodeList nodelist = root.getElementsByTagName("answer");
            for (int i = 0; i < nodelist.getLength(); i++) {
                Element answerNode = (Element) nodelist.item(i);

                //defaultne hodnoty
                String answertext = "";
                int position = -1;
                int code = -1;

                if (answerNode.getElementsByTagName("answertext").item(0).getChildNodes().getLength() != 0) {
                    answertext = answerNode.getElementsByTagName("answertext").item(0).getChildNodes().item(0).getNodeValue();
                }
                if (answerNode.getElementsByTagName("position").item(0).getChildNodes().getLength() != 0) {
                    position = Integer.parseInt(answerNode.getElementsByTagName("position").item(0).getChildNodes().item(0).getNodeValue());
                }
                if (answerNode.getElementsByTagName("code").item(0).getChildNodes().getLength() != 0) {
                    code = Integer.parseInt(answerNode.getElementsByTagName("code").item(0).getChildNodes().item(0).getNodeValue());
                }
                model.addElement(new Answer(answertext, position, code));
            }
        }
    }

    /**
     * Metoda pozbiera informacie z formulara a vytvori info(definujuce otazku) o otazke mborder. Udaje su tiez kontrolovane.
     * Najprv ziska odpovede a precisluje ich podla poradia. 
     * Skontroluje ci je zadany nazov otazky a pocet odpovedi je viac ako tri.
     * Pouziva sa pri generovani vysledneho xml suboru.
     * @return vrati info o otazke
     * @throws ValidationException ak otazka nesplna kriteria
     */
    public OrderDefinition getOrderDefWithValidation() throws ValidationException {
        List anss = new ArrayList();
        for (int i = 0; i < answersList.getModel().getSize(); i++) {
            Answer ans = (Answer) answersList.getModel().getElementAt(i);
            ans.setPosition(i);
            anss.add(ans);
        }
        if (questionNameTextField.getText().isEmpty()) {
            throw new ValidationException("Názov otázky musí byť definovaný!");
        }
        if (anss.size() < 3) {
            throw new ValidationException("Počet možnosti musí byť minimálne 3!");
        }
        return new OrderDefinition(questionNameTextField.getText(), questionTextTextArea.getText(), anss);
    }

    /**
     * Metoda pozbiera informacie z formulara a vytvori info(definujuce otazku) o otazke mborder. Udaje nie su kontrolovane.
     * Pouziva sa pri nahlaiadnuti xml v programe(je to len informativne xml nie je potrebna kontrola). 
     * @return info o otazke
     */
    public OrderDefinition getOrderDefWithoutValidation() {
        List anss = new ArrayList();
        for (int i = 0; i < answersList.getModel().getSize(); i++) {
            Answer ans = (Answer) answersList.getModel().getElementAt(i);
            ans.setPosition(i);
            anss.add(ans);
        }
        return new OrderDefinition(questionNameTextField.getText(), questionTextTextArea.getText(), anss);

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        questionTextLabel = new javax.swing.JLabel();
        answersListScrollPane = new javax.swing.JScrollPane();
        answersList = new ReorderableList(model);
        questionNameLabel = new javax.swing.JLabel();
        questionNameTextField = new javax.swing.JTextField();
        jButton1 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        questionTextTextArea = new javax.swing.JTextArea();

        setBackground(new java.awt.Color(255, 255, 255));

        questionTextLabel.setText("Text otázky :");

        answersList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        answersList.setDropMode(javax.swing.DropMode.INSERT);
        answersListScrollPane.setViewportView(answersList);

        questionNameLabel.setText("Názov otázky :");

        jButton1.setText("+");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jButton3.setText("-");
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        questionTextTextArea.setColumns(20);
        questionTextTextArea.setFont(new java.awt.Font("Arial", 0, 12)); // NOI18N
        questionTextTextArea.setRows(5);
        jScrollPane1.setViewportView(questionTextTextArea);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jButton3, 0, 40, Short.MAX_VALUE)
                            .addComponent(jButton1, javax.swing.GroupLayout.PREFERRED_SIZE, 40, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(answersListScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 327, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(53, 53, 53))
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 426, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(questionNameLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 354, Short.MAX_VALUE))
                    .addComponent(questionNameTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 426, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(questionTextLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 362, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(questionNameLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(questionNameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(questionTextLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jButton1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jButton3))
                    .addComponent(answersListScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 185, Short.MAX_VALUE))
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        DefaultListModel dlm = (DefaultListModel) answersList.getModel();
        Answer item = new Answer(answersList.getComponentCount());
        GetAnswer dialog = new GetAnswer(null, true, item.getAnswer());
        dialog.setVisible(true);
        item.setAnswer(dialog.getValue());
        dlm.addElement(item);
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        int index = answersList.getSelectedIndex();
        if (index != -1) {
            ((DefaultListModel) answersList.getModel()).remove(index);
            answersList.setSelectedIndex(index++);
        } else {
            answersList.setSelectedIndex(0);
        }
    }//GEN-LAST:event_jButton3ActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JList answersList;
    private javax.swing.JScrollPane answersListScrollPane;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JLabel questionNameLabel;
    private javax.swing.JTextField questionNameTextField;
    private javax.swing.JLabel questionTextLabel;
    private javax.swing.JTextArea questionTextTextArea;
    // End of variables declaration//GEN-END:variables
}
