package it4kt.order.graphic.panels;

import exceptions.ValidationException;
import interfaces.IPanel;
import javax.xml.transform.dom.DOMSource;
import it4kt.order.utils.XML;
import org.w3c.dom.Document;

/**
 * Hlavna trieda pre otazku mborder.
 * Spravuje prepinanie medzi designom a nahliadnutim do xml.
 * @author marek
 */
public class MBorderPanel extends javax.swing.JPanel implements IPanel {

    XMLPanel xml_panel;
    DesignPanel design_panel;
    XML XML;

    /**
     * Creates new form mborderPanel
     * Vytvori a nastavi vsetky hodnoty.
     * Ziska url suboru.
     * @param document 
     */
    public MBorderPanel(Document document) {
        initComponents();
        XML = new XML();
        if (document != null && document.getBaseURI() != null) {
            XML.setUrl(document.getBaseURI().substring(6));
        }
        xml_panel = new XMLPanel();
        design_panel = new DesignPanel(document);
        designToggleButton.setSelected(true);
        showDesign();
    }

    /**
     * Metoda zneviditelny xml panel, nastavi velkost design panelu a potom ho zobrazi.
     */
    private void showDesign() {
        mainPanel.remove(xml_panel);
        design_panel.setSize(mainPanel.getSize());
        mainPanel.add(design_panel);
    }

    /**
     * Vrati DOM strukturu otazky
     * @return
     * @throws ValidationException 
     */
    public DOMSource getXMLContent() throws ValidationException {
        return XML.generateXML(design_panel.getOrderDefWithValidation());
    }

    /**
     * Vrati cestu k suboru
     * @return 
     */
    public String getUrlOfFile() {
        return XML.getUrl();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup_xml_design = new javax.swing.ButtonGroup();
        toolBarPanel = new javax.swing.JPanel();
        designToggleButton = new javax.swing.JToggleButton();
        xmlToggleButton = new javax.swing.JToggleButton();
        mainPanelScrollPane = new javax.swing.JScrollPane();
        mainPanel = new javax.swing.JPanel();

        setBackground(new java.awt.Color(255, 255, 255));
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                MBorderPanel.this.componentResized(evt);
            }
        });

        toolBarPanel.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        buttonGroup_xml_design.add(designToggleButton);
        designToggleButton.setText("Design");
        designToggleButton.setBorderPainted(false);
        designToggleButton.setFocusPainted(false);
        designToggleButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                designToggleButtonActionPerformed(evt);
            }
        });

        buttonGroup_xml_design.add(xmlToggleButton);
        xmlToggleButton.setText("XML");
        xmlToggleButton.setBorderPainted(false);
        xmlToggleButton.setMinimumSize(new java.awt.Dimension(64, 22));
        xmlToggleButton.setPreferredSize(new java.awt.Dimension(64, 22));
        xmlToggleButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                xmlToggleButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout toolBarPanelLayout = new javax.swing.GroupLayout(toolBarPanel);
        toolBarPanel.setLayout(toolBarPanelLayout);
        toolBarPanelLayout.setHorizontalGroup(
            toolBarPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(toolBarPanelLayout.createSequentialGroup()
                .addComponent(designToggleButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(xmlToggleButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        toolBarPanelLayout.setVerticalGroup(
            toolBarPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(toolBarPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(designToggleButton, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addComponent(xmlToggleButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        mainPanelScrollPane.setBackground(new java.awt.Color(255, 255, 255));
        mainPanelScrollPane.setBorder(null);

        mainPanel.setBackground(new java.awt.Color(255, 255, 255));
        mainPanel.addContainerListener(new java.awt.event.ContainerAdapter() {
            public void componentAdded(java.awt.event.ContainerEvent evt) {
                MBorderPanel.this.componentAdded(evt);
            }
        });

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 557, Short.MAX_VALUE)
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 409, Short.MAX_VALUE)
        );

        mainPanelScrollPane.setViewportView(mainPanel);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(toolBarPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(mainPanelScrollPane)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(toolBarPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(mainPanelScrollPane))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void xmlToggleButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_xmlToggleButtonActionPerformed
        mainPanel.remove(design_panel);
        DOMSource source = XML.generateXML(design_panel.getOrderDefWithoutValidation());
        xml_panel.setValue(XML.generateStringFromXML(source));
        xml_panel.setSize(mainPanel.getSize());
        mainPanel.add(xml_panel);
    }//GEN-LAST:event_xmlToggleButtonActionPerformed

    private void designToggleButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_designToggleButtonActionPerformed
        showDesign();
    }//GEN-LAST:event_designToggleButtonActionPerformed

    private void componentAdded(java.awt.event.ContainerEvent evt) {//GEN-FIRST:event_componentAdded
        updateUI();
        repaint();
    }//GEN-LAST:event_componentAdded

    private void componentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_componentResized
        xml_panel.setSize(mainPanel.getSize());
        design_panel.setSize(mainPanel.getSize());
        repaint();
    }//GEN-LAST:event_componentResized
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup_xml_design;
    private javax.swing.JToggleButton designToggleButton;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JScrollPane mainPanelScrollPane;
    private javax.swing.JPanel toolBarPanel;
    private javax.swing.JToggleButton xmlToggleButton;
    // End of variables declaration//GEN-END:variables
}
