package it4kt.order.utils;

import it4kt.order.graphic.panels.MBorderPanel;
import interfaces.IModule;
import java.io.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JPanel;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.SchemaFactory;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Hlavna trieda pre otazku mborder. 
 * Implementuje rozhranie IModule
 * @author marek
 */
public class Order implements IModule {

    /**
     * Vrati podporovane koncovky suborov.
     * @return 
     */
    public String[] getExtensions() {
        return new String[]{"xml", "jmx"};
    }

    /**
     * Skontroluje ci subor je mozne spracovat. 
     * Kontrola prebieha najprv porovnanim koncovky suboru.
     * Ak je koncovka jmx(hot potatoes) potom je spravatelna.
     * Ak je koncovka xml porovna sa ci xml vyhovuje scheme pre otazku mborder
     * @param url
     * @return 
     */
    public boolean isFileAcceptable(String url) {
        String extension = url.substring(url.lastIndexOf(".") + 1, url.length());
        if ("jmx".equals(extension)) {
            return true;
        } else if ("xml".equals(extension)) {
            try {
                getDocumentFromFile(url);
                return true;
            } catch (SAXException ex) {
                return false;
            }
        }
        return false;
    }

    /**
     * Ziskanie Documentu z xml suboru.
     * Ak je subor s koncovkou jmx musi sa najprv vykonat xslt transformacia, ktora transformuje jmx do pozadovaneho xml suboru.
     * Potom sa skontroluje ci vyhovuje scheme a xml sa transformuje do dokumentu.
     * Ak je xml potom sa hned kontroluje ci vyhovuje scheme a transformuje sa na dokument.
     * @param url
     * @return
     * @throws SAXException 
     */
    public Document getDocumentFromFile(String url) throws SAXException {
        String extension = url.substring(url.lastIndexOf(".") + 1, url.length());
        try {
            if (extension.equals("jmx")) {
                javax.xml.transform.Source xmlSource =
                        new javax.xml.transform.stream.StreamSource(new File(url));
                javax.xml.transform.Source xsltSource =
                        new javax.xml.transform.stream.StreamSource(getClass().getResourceAsStream("/xslt/mborder.xsl"));
                StreamResult result = new StreamResult(new StringWriter());

                // create an instance of TransformerFactory
                javax.xml.transform.TransformerFactory transFact =
                        javax.xml.transform.TransformerFactory.newInstance();

                javax.xml.transform.Transformer trans =
                        transFact.newTransformer(xsltSource);

                trans.transform(xmlSource, result);
                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                factory.setValidating(false);
                factory.setNamespaceAware(true);
                SchemaFactory schemaFactory =
                        SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");

                factory.setSchema(schemaFactory.newSchema(
                        new Source[]{new StreamSource(getClass().getResourceAsStream("/xsd/mborder.xsd"))}));
                DocumentBuilder builder = factory.newDocumentBuilder();
                return builder.parse(new InputSource(new ByteArrayInputStream(result.getWriter().toString().getBytes())));
            }
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            factory.setNamespaceAware(true);
            SchemaFactory schemaFactory =
                    SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");

            factory.setSchema(schemaFactory.newSchema(
                    new Source[]{new StreamSource(getClass().getResourceAsStream("/xsd/mborder.xsd"))}));
            DocumentBuilder builder = factory.newDocumentBuilder();
            builder.setErrorHandler(new SimpleErrorHandler());
            return builder.parse(new File(url));
        } catch (TransformerException ex) {
            Logger.getLogger(Order.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(Order.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ParserConfigurationException ex) {
            System.out.println("parser exception");
            Logger.getLogger(Order.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    /**
     * vrati kod otazky
     * @return 
     */
    public String getCode() {
        return "mborder";
    }

    /**
     * Vrati nazov otazky
     * @return 
     */
    public String getName() {
        return "Order question";
    }

    /**
     * Vrati ikonu otazky
     * @return 
     */
    public javax.swing.ImageIcon getImage() {
        return new javax.swing.ImageIcon(getClass().getResource("/img/icon.gif"));
    }

    /**
     * Vytvori novu alebo otvori staru otazku.
     * @param document
     * @return 
     */
    public JPanel execute(Document document) {
        return new MBorderPanel(document);
    }
}
