package it4kt.order.utils;

import it4kt.order.data.Answer;
import it4kt.order.data.OrderDefinition;
import java.io.StringWriter;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Trieda, ktora sa stara o vytvaranie xml
 * @author marek
 */
public class XML {

    private String url;

    /**
     * Vrati DOM strukturu otazky
     * @param OrderDefinition
     * @return 
     */
    public DOMSource generateXML(OrderDefinition info) {
        try {
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = documentBuilder.newDocument();

            //question element
            Element question = document.createElementNS("http://xml.moodle.org/schema/it4kt_order", "question");
            document.appendChild(question);

            //name element
            Element name = document.createElement("name");
            name.appendChild(document.createTextNode(info.getName()));
            question.appendChild(name);

            //question text element
            Element questiontext = document.createElement("questiontext");
            questiontext.appendChild(document.createTextNode(info.getQuestiontext()));
            question.appendChild(questiontext);

            //answers element
            Element answers = document.createElement("answers");
            for (int i = 0; i < info.getAnswers().size(); i++) {
                Element answer = document.createElement("answer");
                //answertext element
                Element answertext = document.createElement("answertext");
                Answer ans = (Answer) info.getAnswers().get(i);
                answertext.appendChild(document.createTextNode(ans.getAnswer()));
                answer.appendChild(answertext);
                //position element
                Element position = document.createElement("position");
                position.appendChild(document.createTextNode(String.valueOf(((Answer) info.getAnswers().get(i)).getPosition())));
                answer.appendChild(position);
                //code element
                Element code = document.createElement("code");
                code.appendChild(document.createTextNode(String.valueOf(((Answer) info.getAnswers().get(i)).getCode())));
                answer.appendChild(code);
                answers.appendChild(answer);
            }
            question.appendChild(answers);

            DOMSource source = new DOMSource(document);
            return source;
        } catch (ParserConfigurationException ex) {
            Logger.getLogger(Order.class.getName()).log(Level.SEVERE, null, ex);
            return null;
        }
    }

    /**
     * Vrati textovu reprezentaciu DOM struktury
     * @param source
     * @return 
     */
    public String generateStringFromXML(DOMSource source) {
        try {
            StringWriter stw = new StringWriter();
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            transformerFactory.setAttribute("indent-number", 2);
            Transformer serializer = transformerFactory.newTransformer();
            serializer.setOutputProperty(OutputKeys.INDENT, "yes");
            serializer.transform(source, new StreamResult(stw));
            return stw.toString();
        } catch (TransformerException ex) {
            Logger.getLogger(XML.class.getName()).log(Level.SEVERE, null, ex);
            return "";
        }
    }

    /**
     * @return the url
     */
    public String getUrl() {
        return url;
    }

    /**
     * @param url the url to set
     */
    public void setUrl(String url) {
        this.url = url;
    }
}
