/*
 * DesignPanel.java
 *
 * Created on 20.11.2011, 22:16:17
 */
package it4kt.partcode.graphic.panels;

import it4kt.partcode.data.Answer;
import it4kt.partcode.data.PartCodeDefinitionConnection;
import exceptions.ValidationException;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.swing.JOptionPane;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledEditorKit.BoldAction;
import javax.swing.text.html.HTMLDocument;
import org.w3c.dom.*;

/**
 *
 * @author marek
 */
public class DesignPanelConnection extends javax.swing.JPanel {

    private int i = 0;
    private List<Answer> answers;
    boolean boldChange = true;

    /** Creates new form DesignPanel */
    public DesignPanelConnection(Document document) {
        initComponents();
        answers = new ArrayList<Answer>();
        jTextPane1.setDocument(new MyDocument());
        jTextPane1.setCaretPosition(0);
        jTextPane1.setMargin(new Insets(5, 5, 5, 5));
        initQuestion(document);


    }

    /**
     * Metoda nastavuje hodnoty pre otazku it4kt_partcode pick
     * Ak dokument existuje potom sa nastavia hodnoty otazky z tohto dokumentu.
     * Ak dokument neexistuje potom sa nastavia preddefinovane hodnoty.
     * @param document 
     */
    private void initQuestion(Document document) {
        if (document != null) {
            Element root = document.getDocumentElement();
            if (root.getElementsByTagName("name").item(0).getChildNodes().getLength() != 0) {
                questionNameTextField.setText(root.getElementsByTagName("name").item(0).getChildNodes().item(0).getNodeValue());
            }
            if (root.getElementsByTagName("questiontext").item(0).getChildNodes().getLength() != 0) {
                textOtazkyTextArea.setText(root.getElementsByTagName("questiontext").item(0).getChildNodes().item(0).getNodeValue());
            }
            if (root.getElementsByTagName("code").item(0).getChildNodes().getLength() != 0) {
                String code = root.getElementsByTagName("code").item(0).getChildNodes().item(0).getNodeValue();
                String[] codeSplit = code.split("\\$partcode-replace[\\d]*");
                code = codeSplit[0];
                for (int i = 1; i < codeSplit.length; i++) {
                    code = code + "<b>$partcode-replace" + (i - 1) + "</b>" + codeSplit[i];
                }
                jTextPane1.setText("<pre>" + code + "</pre>");
            }
            if (root.getElementsByTagName("check_service").item(0).getChildNodes().getLength() != 0) {
                jTextField1.setText(root.getElementsByTagName("check_service").item(0).getChildNodes().item(0).getNodeValue());
            }
            if (root.getElementsByTagName("control_wsdl").item(0).getChildNodes().getLength() != 0) {
                jTextField2.setText(root.getElementsByTagName("control_wsdl").item(0).getChildNodes().item(0).getNodeValue());
            }
            if (root.getElementsByTagName("control_method").item(0).getChildNodes().getLength() != 0) {
                jTextField3.setText(root.getElementsByTagName("control_method").item(0).getChildNodes().item(0).getNodeValue());
            }
            if (root.getElementsByTagName("control_param_name").item(0).getChildNodes().getLength() != 0) {
                jTextField4.setText(root.getElementsByTagName("control_param_name").item(0).getChildNodes().item(0).getNodeValue());
            }

        }
    }

    /**
     * Metoda pozbiera informacie z formulara a vytvori info(definujuce otazku) o otazke mbimage pick. Udaje su tiez kontrolovane.
     * Skontroluje ci su udaje v poriadku. 
     * Pouziva sa pri generovani vysledneho xml suboru.
     * @return vrati info o otazke
     * @throws ValidationException ak otazka nesplna kriteria
     */
    public PartCodeDefinitionConnection getConnectionDefinition() throws ValidationException {
        String code = renumberingAnswers();
        if (questionNameTextField.getText().isEmpty()) {
            throw new ValidationException("Nazov otazky musi byt definovany!");
        }
        if (jTextField1.getText().isEmpty()) {
            throw new ValidationException("Syntaxovy servlet musi byt definovany!");
        }
        if (jTextField2.getText().isEmpty()) {
            throw new ValidationException("WSDL sluzby musi byt definovana!");
        }
        if (jTextField3.getText().isEmpty()) {
            throw new ValidationException("Metoda sluzby musi byt definovana!");
        }
        if (jTextField4.getText().isEmpty()) {
            throw new ValidationException("Nazov parametra musi byt definovany!");
        }
        return new PartCodeDefinitionConnection(questionNameTextField.getText(), textOtazkyTextArea.getText(), code, jTextField1.getText(), jTextField2.getText(), jTextField3.getText(), jTextField4.getText());
    }

    /**
     * Metoda pozbiera informacie z formulara a vytvori info(definujuce otazku) o otazke mbimage pick. Udaje nie su kontrolovane.
     * Pouziva sa pri nahlaiadnuti xml v programe(je to len informativne xml nie je potrebna kontrola). 
     * @return info o otazke
     */
    public PartCodeDefinitionConnection getConnectionDef() {
        String code = renumberingAnswers();
        return new PartCodeDefinitionConnection(questionNameTextField.getText(), textOtazkyTextArea.getText(), code, jTextField1.getText(), jTextField2.getText(), jTextField3.getText(), jTextField4.getText());
    }

    private String renumberingAnswers() {
        boldChange = false;
        String code = "";
        List<Answer> reanswers = new ArrayList<Answer>();
        try {
            code = jTextPane1.getDocument().getText(0, jTextPane1.getDocument().getLength());
            for (int i = 0; i < answers.size(); i++) {
                Pattern p = Pattern.compile("(\\$partcode-replace[\\d]*)(.*)");
                Matcher m = p.matcher(code);
                if (m.find()) {
                    for (int j = 0; j < answers.size(); j++) {
                        if (answers.get(j).getCode().equals(m.group(1))) {
                            Answer answer = new Answer("$partcode-replace" + i, answers.get(j).getAnswers());
                            reanswers.add(answer);
                            break;
                        }
                    }
                    code = m.group(2);
                }
            }
            code = jTextPane1.getDocument().getText(0, jTextPane1.getDocument().getLength());
            String[] codeSplit = code.split("\\$partcode-replace[\\d]*");
            code = codeSplit[0];
            String codeHTML = codeSplit[0];
            for (int i = 1; i < codeSplit.length; i++) {
                code = code + "$partcode-replace" + (i - 1) + codeSplit[i];
                codeHTML = codeHTML + "<b>$partcode-replace" + (i - 1) + "</b>" + codeSplit[i];
            }
            jTextPane1.setText("<pre>" + codeHTML + "</pre>");
            Collections.sort(reanswers, new AnswerComparator());
            answers = reanswers;
        } catch (BadLocationException ex) {
            Logger.getLogger(DesignPanelConnection.class.getName()).log(Level.SEVERE, null, ex);
        }
        boldChange = true;
        return code;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        selectionButtonGroup = new javax.swing.ButtonGroup();
        answersCountButtonGroup = new javax.swing.ButtonGroup();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        questionNameTextField = new javax.swing.JTextField();
        jScrollPane3 = new javax.swing.JScrollPane();
        textOtazkyTextArea = new javax.swing.JTextArea();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTextPane1 = new javax.swing.JTextPane();
        jButton1 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        jButton4 = new javax.swing.JButton();
        jTextField1 = new javax.swing.JTextField();
        jTextField2 = new javax.swing.JTextField();
        jTextField3 = new javax.swing.JTextField();
        jTextField4 = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();

        setBackground(new java.awt.Color(255, 255, 255));

        jLabel1.setText("Text otázky :");

        jLabel2.setText("Názov otázky :");

        textOtazkyTextArea.setColumns(20);
        textOtazkyTextArea.setFont(new java.awt.Font("Tahoma", 0, 11)); // NOI18N
        textOtazkyTextArea.setRows(4);
        textOtazkyTextArea.setText("sadasda");
        jScrollPane3.setViewportView(textOtazkyTextArea);

        jTextPane1.setContentType("text/html"); // NOI18N
        jScrollPane1.setViewportView(jTextPane1);

        jButton1.setText("Prázdne miesto");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jButton3.setText("Zmaž prázdne miesto");
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        jButton4.setText("Zmaž všetky prázdne miesta");
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });

        jLabel3.setText("Syntaxovy servlet:");

        jLabel4.setText("Vyhodnocovacia služba:");

        jLabel5.setText("Meno služby:");

        jLabel6.setText("Názov parametra služby:");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 1106, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addComponent(jScrollPane3, javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel2, javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(jLabel1)
                                .addGap(316, 316, 316))
                            .addComponent(questionNameTextField, javax.swing.GroupLayout.Alignment.LEADING))
                        .addGap(26, 26, 26)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jButton1, javax.swing.GroupLayout.PREFERRED_SIZE, 169, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addComponent(jButton4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(jButton3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                        .addGap(29, 29, 29)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel4)
                            .addComponent(jLabel3)
                            .addComponent(jLabel5)
                            .addComponent(jLabel6))
                        .addGap(13, 13, 13)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(jTextField4, javax.swing.GroupLayout.DEFAULT_SIZE, 369, Short.MAX_VALUE)
                            .addComponent(jTextField2, javax.swing.GroupLayout.DEFAULT_SIZE, 369, Short.MAX_VALUE)
                            .addComponent(jTextField3, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 369, Short.MAX_VALUE)
                            .addComponent(jTextField1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 369, Short.MAX_VALUE))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel2)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE, false)
                            .addComponent(questionNameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jButton1))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(jLabel1)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 55, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(jButton3, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                    .addComponent(jButton4, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addComponent(jLabel6)
                                    .addComponent(jTextField4, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)))))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel3)
                            .addComponent(jTextField1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel4)
                            .addComponent(jTextField2, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel5)
                            .addComponent(jTextField3, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 415, Short.MAX_VALUE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        if (jTextPane1.getSelectionStart() != jTextPane1.getSelectionEnd()) {
            boldChange = false;
            String code = "$partcode-replace" + i++;
            new BoldAction().actionPerformed(evt);
            answers.add(new Answer(code, jTextPane1.getSelectedText()));
            jTextPane1.replaceSelection(code);
            boldChange = true;
        } else {
            JOptionPane.showMessageDialog(this, "Musi byt oznaceny text.", "Information", JOptionPane.INFORMATION_MESSAGE);
        }
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
        try {
            boldChange = false;
            String codeHTML = jTextPane1.getDocument().getText(0, jTextPane1.getDocument().getLength());
            for (int i = 0; i < answers.size(); i++) {
                codeHTML = codeHTML.replace(answers.get(i).getCode(), answers.get(i).getAnswers().get(0).trim());
            }
            String[] codeSplit = codeHTML.split("\\$partcode-replace[\\d]*");
            codeHTML = codeSplit[0];
            for (int j = 1; j < codeSplit.length; j++) {
                codeHTML = codeHTML + "<b>$partcode-replace" + (j - 1) + "</b>" + codeSplit[j];
            }
            jTextPane1.setText("<pre>" + codeHTML + "</pre>");
            answers = new ArrayList<Answer>();
            boldChange = true;
        } catch (BadLocationException ex) {
            Logger.getLogger(DesignPanelConnection.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_jButton4ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        javax.swing.text.Element element = jTextPane1.getStyledDocument().getCharacterElement(jTextPane1.getSelectionStart());
        AttributeSet set = element.getAttributes();
        if (StyleConstants.isBold(set) && jTextPane1.getSelectionStart() == jTextPane1.getSelectionEnd()) {
            int start = jTextPane1.getSelectionStart();
            while (StyleConstants.isBold(set)) {
                element = jTextPane1.getStyledDocument().getCharacterElement(start--);
                set = element.getAttributes();
            }
            start++;
            int end = jTextPane1.getSelectionEnd();
            element = jTextPane1.getStyledDocument().getCharacterElement(end);
            set = element.getAttributes();
            while (StyleConstants.isBold(set)) {
                element = jTextPane1.getStyledDocument().getCharacterElement(end++);
                set = element.getAttributes();
            }
            end--;
            try {
                boldChange = false;
                for (int i = 0; i < answers.size(); i++) {
                    if (answers.get(i).getCode().equals(jTextPane1.getText(start, end - start).trim())) {
                        String codeHTML = jTextPane1.getDocument().getText(0, jTextPane1.getDocument().getLength());
                        codeHTML = codeHTML.replace(answers.get(i).getCode(), answers.get(i).getAnswers().get(0).trim());
                        
                        jTextPane1.setText("<pre>" + codeHTML + "</pre>");
                        answers.remove(i);
                        renumberingAnswers();
                        break;
                    }
                }
                boldChange = true;
            } catch (BadLocationException ex) {
                Logger.getLogger(DesignPanelConnection.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else {
            JOptionPane.showMessageDialog(this, "Musi byt oznacena diera.", "Information", JOptionPane.INFORMATION_MESSAGE);
        }
    }//GEN-LAST:event_jButton3ActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup answersCountButtonGroup;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JTextField jTextField2;
    private javax.swing.JTextField jTextField3;
    private javax.swing.JTextField jTextField4;
    private javax.swing.JTextPane jTextPane1;
    private javax.swing.JTextField questionNameTextField;
    private javax.swing.ButtonGroup selectionButtonGroup;
    private javax.swing.JTextArea textOtazkyTextArea;
    // End of variables declaration//GEN-END:variables

    class MyDocument extends HTMLDocument {

        @Override
        public void remove(int offs, int len) throws BadLocationException {
            if (boldChange) {
                boolean isBold = false;
                for (int i = offs; i < offs + len; i++) {
                    javax.swing.text.Element element = jTextPane1.getStyledDocument().getCharacterElement(i);
                    AttributeSet set = element.getAttributes();
                    if (StyleConstants.isBold(set)) {
                        isBold = true;
                        break;
                    }
                }
                if (!isBold) {
                    super.remove(offs, len);
                }
            } else {
                super.remove(offs, len);
            }
        }

        @Override
        public void insertString(int offset, String str, AttributeSet a) throws BadLocationException {
            if (boldChange) {
                javax.swing.text.Element element = jTextPane1.getStyledDocument().getCharacterElement(offset);
                AttributeSet set = element.getAttributes();
                if (!StyleConstants.isBold(set)) {
                    super.insertString(offset, str, a);
                }
            } else {
                super.insertString(offset, str, a);
            }
        }
    }

    class AnswerComparator implements Comparator {

        public int compare(Object ans1, Object ans2) {

            /*
             * parameter are of type Object, so we have to downcast it
             * to Employee objects
             */

            String ans1Code = ((Answer) ans1).getCode();
            String ans2Code = ((Answer) ans2).getCode();

            if (ans1Code.compareTo(ans2Code) > 0) {
                return 1;
            } else if (ans1Code.compareTo(ans2Code) < 0) {
                return -1;
            } else {
                return 0;
            }
        }
    }
}
