/*
 * DesignPanel.java
 *
 * Created on 20.11.2011, 22:16:17
 */
package it4kt.partcode.graphic.panels;

import it4kt.partcode.data.Answer;
import exceptions.ValidationException;
import it4kt.partcode.data.PartCodeDefinitionText;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.swing.JOptionPane;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledEditorKit.BoldAction;
import javax.swing.text.html.HTMLDocument;
import org.w3c.dom.*;

/**
 *
 * @author marek
 */
public class DesignPanelText extends javax.swing.JPanel {

    private int i = 0;
    private List<Answer> answers;
    boolean boldChange = true;

    /** Creates new form DesignPanel */
    public DesignPanelText(Document document) {
        initComponents();
        answers = new ArrayList<Answer>();
        jTextPane1.setDocument(new MyDocument());
        jTextPane1.setCaretPosition(0);
        jTextPane1.setMargin(new Insets(5, 5, 5, 5));
        initQuestion(document);


    }

    /**
     * Metoda nastavuje hodnoty pre otazku it4kt_partcode pick
     * Ak dokument existuje potom sa nastavia hodnoty otazky z tohto dokumentu.
     * Ak dokument neexistuje potom sa nastavia preddefinovane hodnoty.
     * @param document 
     */
    private void initQuestion(Document document) {
        if (document != null) {
            Element root = document.getDocumentElement();
            if (root.getElementsByTagName("name").item(0).getChildNodes().getLength() != 0) {
                questionNameTextField.setText(root.getElementsByTagName("name").item(0).getChildNodes().item(0).getNodeValue());
            }
            if (root.getElementsByTagName("questiontext").item(0).getChildNodes().getLength() != 0) {
                textOtazkyTextArea.setText(root.getElementsByTagName("questiontext").item(0).getChildNodes().item(0).getNodeValue());
            }
            if (root.getElementsByTagName("code").item(0).getChildNodes().getLength() != 0) {
                String code = root.getElementsByTagName("code").item(0).getChildNodes().item(0).getNodeValue();
                code = code.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
                String[] codeSplit = code.split("\\$partcode-replace[\\d]*");
                code = codeSplit[0];
                for (i = 0; i < codeSplit.length - 1; i++) {
                    code = code + "<b>$partcode-replace" + i + "</b>" + codeSplit[i+1];

                    List<String> anss = new ArrayList<String>();
                    NodeList nodelist = root.getElementsByTagName("answers");
                    for (int j = 0; j < nodelist.getLength(); j++) {
                        if (i == j) {
                            Element answerNode = (Element) nodelist.item(j);
                            NodeList answerList = answerNode.getElementsByTagName("answer");
                            for (int k = 0; k < answerList.getLength(); k++) {
                                Element answer = (Element) answerList.item(k);
                                anss.add(answer.getChildNodes().item(0).getNodeValue());
                            }
                        }
                    }
                    answers.add(new Answer("$partcode-replace" + i, anss));
                }
                code = "<pre>" + code + "</pre>";
                jTextPane1.setText(code);
            }

        }
    }

    /**
     * Metoda pozbiera informacie z formulara a vytvori info(definujuce otazku) o otazke mbimage pick. Udaje su tiez kontrolovane.
     * Skontroluje ci su udaje v poriadku. 
     * Pouziva sa pri generovani vysledneho xml suboru.
     * @return vrati info o otazke
     * @throws ValidationException ak otazka nesplna kriteria
     */
    public PartCodeDefinitionText getTextDefinition() throws ValidationException {
        String code = renumberingAnswers();
        if (questionNameTextField.getText().isEmpty()) {
            throw new ValidationException("Nazov otazky musi byt definovany!");
        }
        if (answers.size() < 1) {
            throw new ValidationException("Pocet odpovedi musi byt minimalne 1!");
        }
        return new PartCodeDefinitionText(questionNameTextField.getText(), textOtazkyTextArea.getText(), code, answers);
    }

    /**
     * Metoda pozbiera informacie z formulara a vytvori info(definujuce otazku) o otazke mbimage pick. Udaje nie su kontrolovane.
     * Pouziva sa pri nahlaiadnuti xml v programe(je to len informativne xml nie je potrebna kontrola). 
     * @return info o otazke
     */
    public PartCodeDefinitionText getTextDef() {
        String code = renumberingAnswers();
        return new PartCodeDefinitionText(questionNameTextField.getText(), textOtazkyTextArea.getText(), code, answers);
    }

    private String renumberingAnswers() {
        boldChange = false;
        String code = "";
        List<Answer> reanswers = new ArrayList<Answer>();
        try {
            code = jTextPane1.getDocument().getText(0, jTextPane1.getDocument().getLength());
            code = code.replaceAll("\n", " ");
            for (i = 0; i < answers.size(); i++) {
                Pattern p = Pattern.compile("(\\$partcode-replace[\\d]*)(.*)");
                Matcher m = p.matcher(code);
                if (m.find()) {
                    for (int j = 0; j < answers.size(); j++) {
                        if (answers.get(j).getCode().equals(m.group(1))) {
                            Answer answer = new Answer("$partcode-replace" + i, answers.get(j).getAnswers());
                            reanswers.add(answer);
                            break;
                        }
                    }
                    code = m.group(2);
                }
            }
            code = jTextPane1.getDocument().getText(0, jTextPane1.getDocument().getLength());
            String[] codeSplit = code.split("\\$partcode-replace[\\d]*");
            code = codeSplit[0];
            String codeHTML = codeSplit[0].replaceAll("<", "&lt;").replaceAll(">", "&gt;");
            for (i = 0; i < codeSplit.length - 1; i++) {
                code = code + "$partcode-replace" + i + codeSplit[i+1];
                codeHTML = codeHTML + "<b>$partcode-replace" + i + "</b>" + codeSplit[i+1].replaceAll("<", "&lt;").replaceAll(">", "&gt;");
            }            
            jTextPane1.setText("<pre>" + codeHTML + "</pre>");
            Collections.sort(reanswers, new AnswerComparator());
            answers = reanswers;
        } catch (BadLocationException ex) {
            Logger.getLogger(DesignPanelText.class.getName()).log(Level.SEVERE, null, ex);
        }
        boldChange = true;
        return code;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        selectionButtonGroup = new javax.swing.ButtonGroup();
        answersCountButtonGroup = new javax.swing.ButtonGroup();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        questionNameTextField = new javax.swing.JTextField();
        jScrollPane3 = new javax.swing.JScrollPane();
        textOtazkyTextArea = new javax.swing.JTextArea();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTextPane1 = new javax.swing.JTextPane();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        jButton4 = new javax.swing.JButton();

        setBackground(new java.awt.Color(255, 255, 255));

        jLabel1.setText("Text otázky :");

        jLabel2.setText("Názov otázky :");

        textOtazkyTextArea.setColumns(20);
        textOtazkyTextArea.setFont(new java.awt.Font("Tahoma", 0, 11)); // NOI18N
        textOtazkyTextArea.setRows(5);
        jScrollPane3.setViewportView(textOtazkyTextArea);

        jTextPane1.setContentType("text/html"); // NOI18N
        jScrollPane1.setViewportView(jTextPane1);

        jButton1.setText("Prázdne miesto");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jButton2.setText("Zobraz prázdne miesta");
        jButton2.setActionCommand("Nastav prazdne miesto");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        jButton3.setText("Zmaž prázdne miesto");
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        jButton4.setText("Zmaž všetky prázdne miesta");
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(jLabel2, javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel1)
                        .addGap(316, 316, 316))
                    .addComponent(questionNameTextField, javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane3))
                .addGap(26, 26, 26)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jButton4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jButton3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jButton2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jButton1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap(163, Short.MAX_VALUE))
            .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 748, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(questionNameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButton1))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jButton2)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jButton3)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jButton4)))
                .addGap(49, 49, 49)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 366, Short.MAX_VALUE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        if (jTextPane1.getSelectionStart() != jTextPane1.getSelectionEnd()) {
            boldChange = false;
            String code = "$partcode-replace" + i++;
            new BoldAction().actionPerformed(evt);
            Alternative alt = new Alternative(null, true, new Answer(code, jTextPane1.getSelectedText()));
            alt.setVisible(true);
            answers.add(alt.getValue());
            jTextPane1.replaceSelection(code);
            boldChange = true;
        } else {
            JOptionPane.showMessageDialog(this, "Musi byt oznaceny text.", "Information", JOptionPane.INFORMATION_MESSAGE);
        }
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        javax.swing.text.Element element = jTextPane1.getStyledDocument().getCharacterElement(jTextPane1.getSelectionStart());
        AttributeSet set = element.getAttributes();
        if (StyleConstants.isBold(set) && jTextPane1.getSelectionStart() == jTextPane1.getSelectionEnd()) {
            int start = jTextPane1.getSelectionStart();
            while (StyleConstants.isBold(set)) {
                element = jTextPane1.getStyledDocument().getCharacterElement(start--);
                set = element.getAttributes();
            }
            start++;
            int end = jTextPane1.getSelectionEnd();
            element = jTextPane1.getStyledDocument().getCharacterElement(end);
            set = element.getAttributes();
            while (StyleConstants.isBold(set)) {
                element = jTextPane1.getStyledDocument().getCharacterElement(end++);
                set = element.getAttributes();
            }
            end--;
            try {
                boldChange = false;
                for (int i = 0; i < answers.size(); i++) {
                    if (answers.get(i).getCode().equals(jTextPane1.getText(start+1, end - start-1).trim())) {
                        Alternative alt = new Alternative(null, true, answers.get(i));
                        alt.setVisible(true);
                        break;
                    }
                }
                boldChange = true;
            } catch (BadLocationException ex) {
                Logger.getLogger(DesignPanelText.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else {
            JOptionPane.showMessageDialog(this, "Musi byt oznacena diera.", "Information", JOptionPane.INFORMATION_MESSAGE);
        }
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
        try {
            boldChange = false;
            String codeHTML = jTextPane1.getDocument().getText(0, jTextPane1.getDocument().getLength());
            for (int i = 0; i < answers.size(); i++) {
                codeHTML = codeHTML.replace(answers.get(i).getCode(), answers.get(i).getAnswers().get(0).trim());
            }
            codeHTML = codeHTML.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
//            String[] codeSplit = codeHTML.split("\\$partcode-replace[\\d]*");
//            codeHTML = codeSplit[0];
//            for (int j = 1; j < codeSplit.length; j++) {
//                codeHTML = codeHTML + "<b>$partcode-replace" + (j - 1) + "</b>" + codeSplit[j];
//            }
            jTextPane1.setText("<pre>" + codeHTML + "</pre>");
            answers = new ArrayList<Answer>();
            i = 0;
            boldChange = true;
        } catch (BadLocationException ex) {
            Logger.getLogger(DesignPanelText.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_jButton4ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        javax.swing.text.Element element = jTextPane1.getStyledDocument().getCharacterElement(jTextPane1.getSelectionStart());
        AttributeSet set = element.getAttributes();
        if (StyleConstants.isBold(set) && jTextPane1.getSelectionStart() == jTextPane1.getSelectionEnd()) {
            int start = jTextPane1.getSelectionStart();
            while (StyleConstants.isBold(set)) {
                element = jTextPane1.getStyledDocument().getCharacterElement(start--);
                set = element.getAttributes();
            }
            start++;
            int end = jTextPane1.getSelectionEnd();
            element = jTextPane1.getStyledDocument().getCharacterElement(end);
            set = element.getAttributes();
            while (StyleConstants.isBold(set)) {
                element = jTextPane1.getStyledDocument().getCharacterElement(end++);
                set = element.getAttributes();
            }
            end--;
            try {
                boldChange = false;
                for (int i = 0; i < answers.size(); i++) {
                    if (answers.get(i).getCode().equals(jTextPane1.getText(start+1, end - start - 1).trim())) {
                        String codeHTML = jTextPane1.getDocument().getText(0, jTextPane1.getDocument().getLength());
                        codeHTML = codeHTML.replace(answers.get(i).getCode(), answers.get(i).getAnswers().get(0).trim());
                        codeHTML = codeHTML.replaceAll("<", "&lt;").replaceAll(">", "&gt;");

                        jTextPane1.setText("<pre>" + codeHTML + "</pre>");
                        answers.remove(i);
                        renumberingAnswers();
                        break;
                    }
                }
                boldChange = true;
            } catch (BadLocationException ex) {
                Logger.getLogger(DesignPanelText.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else {
            JOptionPane.showMessageDialog(this, "Musi byt oznacena diera.", "Information", JOptionPane.INFORMATION_MESSAGE);
        }
    }//GEN-LAST:event_jButton3ActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup answersCountButtonGroup;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JTextPane jTextPane1;
    private javax.swing.JTextField questionNameTextField;
    private javax.swing.ButtonGroup selectionButtonGroup;
    private javax.swing.JTextArea textOtazkyTextArea;
    // End of variables declaration//GEN-END:variables

    class MyDocument extends HTMLDocument {

        @Override
        public void remove(int offs, int len) throws BadLocationException {
            if (boldChange) {
                boolean isBold = false;
                for (int i = offs; i < offs + len; i++) {
                    javax.swing.text.Element element = jTextPane1.getStyledDocument().getCharacterElement(i);
                    AttributeSet set = element.getAttributes();
                    if (StyleConstants.isBold(set)) {
                        isBold = true;
                        break;
                    }
                }
                if (!isBold) {
                    super.remove(offs, len);
                }
            } else {
                super.remove(offs, len);
            }
        }

        @Override
        public void insertString(int offset, String str, AttributeSet a) throws BadLocationException {
            if (boldChange) {
                javax.swing.text.Element element = jTextPane1.getStyledDocument().getCharacterElement(offset);
                AttributeSet set = element.getAttributes();
                if (!StyleConstants.isBold(set)) {
                    super.insertString(offset, str, a);
                }
            } else {
                super.insertString(offset, str, a);
            }
        }
    }

    class AnswerComparator implements Comparator {

        public int compare(Object ans1, Object ans2) {

            /*
             * parameter are of type Object, so we have to downcast it
             * to Employee objects
             */

            String ans1Code = ((Answer) ans1).getCode();
            String ans2Code = ((Answer) ans2).getCode();

            if (ans1Code.compareTo(ans2Code) > 0) {
                return 1;
            } else if (ans1Code.compareTo(ans2Code) < 0) {
                return -1;
            } else {
                return 0;
            }
        }
    }
}
