package it4kt.partcode.utils;

import it4kt.partcode.data.Answer;
import it4kt.partcode.data.PartCodeDefinitionConnection;
import it4kt.partcode.data.PartCodeDefinitionText;
import java.io.StringWriter;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Trieda, ktora sa stara o vytvaranie xml
 * @author marek
 */
public class XML {

    private String url;

    /**
     * Vrati DOM strukturu otazky
     * @param OrderDefinition
     * @return 
     */
    public DOMSource generateXML(PartCodeDefinitionConnection info) {
        try {
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = documentBuilder.newDocument();

            //question element
            Element question = document.createElementNS("http://xml.moodle.org/schema/it4kt_partcode", "question");
            document.appendChild(question);

            //name element
            Element name = document.createElement("name");
            name.appendChild(document.createTextNode(info.getName()));
            question.appendChild(name);

            //question text element
            Element questiontext = document.createElement("questiontext");
            questiontext.appendChild(document.createTextNode(info.getQuestiontext()));
            question.appendChild(questiontext);

            //question text element
            Element type = document.createElement("partcodetype");
            type.appendChild(document.createTextNode("connections"));
            question.appendChild(type);

            //question text element
            Element conn = document.createElement("connections");

            //question text element
            Element code = document.createElement("code");
            code.appendChild(document.createTextNode(info.getCode()));
            conn.appendChild(code);

            Element check_service = document.createElement("check_service");
            check_service.appendChild(document.createTextNode(info.getCheck_service()));
            conn.appendChild(check_service);

            Element control_wsdl = document.createElement("control_wsdl");
            control_wsdl.appendChild(document.createTextNode(info.getControl_wsdl()));
            conn.appendChild(control_wsdl);

            Element control_method = document.createElement("control_method");
            control_method.appendChild(document.createTextNode(info.getControl_method()));
            conn.appendChild(control_method);

            Element control_param_name = document.createElement("control_param_name");
            control_param_name.appendChild(document.createTextNode(info.getControl_param_name()));
            conn.appendChild(control_param_name);

            question.appendChild(conn);

            DOMSource source = new DOMSource(document);
            return source;
        } catch (ParserConfigurationException ex) {
            Logger.getLogger(Partcode.class.getName()).log(Level.SEVERE, null, ex);
            return null;
        }
    }

    /**
     * Vrati DOM strukturu otazky
     * @param OrderDefinition
     * @return 
     */
    public DOMSource generateXML(PartCodeDefinitionText info) {
        try {
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = documentBuilder.newDocument();

            //question element
            Element question = document.createElementNS("http://xml.moodle.org/schema/it4kt_partcode", "question");
            document.appendChild(question);

            //name element
            Element name = document.createElement("name");
            name.appendChild(document.createTextNode(info.getName()));
            question.appendChild(name);

            //question text element
            Element questiontext = document.createElement("questiontext");
            questiontext.appendChild(document.createTextNode(info.getQuestiontext()));
            question.appendChild(questiontext);

            //question text element
            Element type = document.createElement("partcodetype");
            type.appendChild(document.createTextNode("text"));
            question.appendChild(type);

            //question text element
            Element text = document.createElement("text");

            //question text element
            Element code = document.createElement("code");
            code.appendChild(document.createTextNode(info.getCode()));
            text.appendChild(code);

            //question text element
            Element replace = document.createElement("replace");
            for (int i = 0; i < info.getAnswers().size(); i++) {
                List alternative = ((Answer) info.getAnswers().get(i)).getAnswers();
                //answers element
                Element answers = document.createElement("answers");
                for (int j = 0; j < alternative.size(); j++) {
                    Element answer = document.createElement("answer");
                    answer.appendChild(document.createTextNode((String) alternative.get(j)));
                    answers.appendChild(answer);
                }
                replace.appendChild(answers);
            }
            text.appendChild(replace);
            question.appendChild(text);

            DOMSource source = new DOMSource(document);
            return source;
        } catch (ParserConfigurationException ex) {
            Logger.getLogger(Partcode.class.getName()).log(Level.SEVERE, null, ex);
            return null;
        }
    }

    /**
     * Vrati textovu reprezentaciu DOM struktury
     * @param source
     * @return 
     */
    public String generateStringFromXML(DOMSource source) {
        try {
            StringWriter stw = new StringWriter();
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            transformerFactory.setAttribute("indent-number", 2);
            Transformer serializer = transformerFactory.newTransformer();
            serializer.setOutputProperty(OutputKeys.INDENT, "yes");
            serializer.transform(source, new StreamResult(stw));
            return stw.toString();
        } catch (TransformerException ex) {
            Logger.getLogger(XML.class.getName()).log(Level.SEVERE, null, ex);
            return "";
        }
    }

    /**
     * @return the url
     */
    public String getUrl() {
        return url;
    }

    /**
     * @param url the url to set
     */
    public void setUrl(String url) {
        this.url = url;
    }
}
