<?php
/**
 * The question type class for the Order question type.
 *
 * @copyright &copy; 2012 Peter Vaclavik
 * @author peter.vaclavik@tuke.sk
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 * @package it4kt
 *//** */

/**
 * The Order question class
 *
 * TODO give an overview of how the class works here.
 */
class it4kt_order_qtype extends default_questiontype {

    function name() {
        return 'it4kt_order';
    }

	/**
     * Allow question types to include required JavaScrip or CSS on pages
     * where they are going to be printed.
     *
     * @param object $question The question object.
     * @param object $state    The state object.
     */
    function get_html_head_contributions(&$question, &$state) {
        global $CFG;
        if (file_exists($CFG->dirroot . '/lib/js/jquery-1.7.1.js')) {
            require_js($CFG->wwwroot . '/lib/js/jquery-1.7.1.js');
        }
        if (file_exists($CFG->dirroot . '/lib/js/ui/jquery.ui.core.js')) {
            require_js($CFG->wwwroot . '/lib/js/ui/jquery.ui.core.js');
        }
        if (file_exists($CFG->dirroot . '/lib/js/ui/jquery.ui.widget.js')) {
            require_js($CFG->wwwroot . '/lib/js/ui/jquery.ui.widget.js');
        }
        if (file_exists($CFG->dirroot . '/lib/js/ui/jquery.ui.mouse.js')) {
            require_js($CFG->wwwroot . '/lib/js/ui/jquery.ui.mouse.js');
        }
        if (file_exists($CFG->dirroot . '/lib/js/ui/jquery.ui.sortable.js')) {
            require_js($CFG->wwwroot . '/lib/js/ui/jquery.ui.sortable.js');
        }
        if (file_exists($CFG->dirroot . '/lib/js/jquery-1.7.1.js')) {
            require_js($CFG->wwwroot . '/lib/js/jquery-1.7.1.js');
        }
        
        $contributions = parent::get_html_head_contributions($question, $state);
        $this->loger('get_html_head_contributions - CSS: ' . implode("\n   ", $contributions));
//        $extra_css = $CFG->wwwroot.'/theme/SUPA4/supa_report.css';
//$extra_css_meta = '<link rel="stylesheet" type="text/css" href="'.$extra_css.'" />';
//print_header_simple('', '', '', '', '', true, '', '', '', $contributions);

        $baseurl = $this->plugin_baseurl();
        echo '<link rel="stylesheet" type="text/css" href="' . $baseurl . '/styles.css" />';
        return $contributions;
    }
    
    // TODO think about whether you need to override the is_manual_graded or
    // is_usable_by_random methods form the base class. Most the the time you
    // Won't need to.

    /**
     * Loads the question type specific options for the question.
     *
     * This function loads any question type specific options for the
     * question from xml file into the question object. This information
     * is placed in the $question->options field. 
	 *
     * @return bool            Indicates success or failure.
     * @param object $question The question object for the question. 
     */
    function get_question_options(&$question) {
        // TODO code to retrieve the extra data you stored in the database into
        // $question->options.
        $this->loger('get_question_options - START');
        global $CFG,$COURSE;
        //global $DB, $OUTPUT;
		
        //get context of the question
        $contextid = get_field('question_categories', 'contextid', 'id', $question->category); 
        $context = get_context_instance_by_id($contextid);
//        $context = $this->get_context_by_category_id($question->category);

        //get file from file storage
        $question_filename = $CFG->dataroot . '/question/' . $COURSE->id . '/' . $question->id . '/' . $question->id;
        $this->loger('get_question_options - laoding XML content from file: \'' . $question_filename . '\'');
        $content = $this->get_file_content($question_filename);

//        $fs = get_file_storage();
//        $files = $fs->get_area_files($context->id, 'qtype_mborder', 'question', $question->id);
//
//        foreach ($files as $file) {
//        $targetfile = $file;
//        }
//        
        //create DOM structure from xml
        $xml = simplexml_load_string($content);
//        $xml = simplexml_load_string($targetfile->get_content());
        
        $json = json_encode($xml);
        $array = json_decode($json,TRUE);     
        //file parsing
        $answers = array();
        $i = 0;
        foreach($array['answers']['answer'] as $a){
                $answers[$i]->answertext = $a['answertext'];
                $answers[$i]->orderposition = $a['position'];
                $answers[$i]->code = $a['code'];			
                $i++;
        }
        $question->options->answers = $answers;     

        $this->loger('get_question_options - END'); 
        return true;
    }

    /**
     * Save the units and the answers associated with this question.
     * @return boolean to indicate success of failure.
     */
    function save_question_options($question) {
        // TODO code to save the extra data to your database tables from the
        // $question object, which has all the post data from editquestion.html
        $this->loger('save_question_options - START');
        
        $context = $question->context;
       	//remove file if exist(edit mode)
        $this->delete_question($question->id,$context->id);

        //set up attibuttes which is needed for file saving	
        $save_file['text']="";
        $save_file['format']= FORMAT_HTML;
        $save_file['itemid']= $question->xmlfile;
        //save file
        $this->import_or_save_files($save_file, $context, 'qtype_it4kt_order', 'question', $question->id);

        $this->loger('save_question_options - END');        
        return true;
    }

    /**
     * Deletes question from the question-type specific tables
     *
     * @param integer $questionid The question being deleted
     * @return boolean to indicate success of failure.
     */
    function delete_question($questionid) {
        global $CFG, $COURSE;
        
        // TODO delete any
        $this->loger('delete_question - START');
        
        $questiondir = $CFG->dataroot . '/question/' . $COURSE->id . '/'. $questionid;
        fulldelete($questiondir);
        
        $this->loger('delete_question - END');
        return true;
    }

    /**
     * Creates empty session and response information for the question
     *
     * This function is called to start a question session. Empty question type
     * specific session data (if any) and created response data will be added to the
     * state object. Session data is any data which must persist throughout the
     * attempt possibly with updates as the user interacts with the
     * question. This function does NOT create new entries in the database for
     * the session; a call to the {@link save_session_and_responses} member will
     * occur to do this.
     * @return bool            Indicates success or failure.
     * @param object $question The question for which the session is to be
     *                         created. Question type specific information is
     *                         included.
     * @param object $state    The state to create the session for. Note that
     *                         this will not have been saved in the database so
     *                         there will be no id. This object will be updated
     *                         to include the question type specific information
     *                         (it is passed by reference). New
     *                         responses will be created in the ->responses
     *                         field.
     * @param object $cmoptions
     * @param object $attempt  The attempt for which the session is to be
     *                         started. Questions may wish to initialize the
     *                         session in different ways depending on the user id
     *                         or time available for the attempt.
     */
    function create_session_and_responses(&$question, &$state, $cmoptions, $attempt) {
        // TODO create a blank repsonse in the $state->responses array, which
        // represents the situation before the student has made a response.
        $this->loger('create_session_and_responses - START');
        
        // Place the questions into the state options keyed by code
        foreach ($question->options->answers as $answer) {
            $state->options->answers[$answer->code] = $answer;
        }
        foreach ($state->options->answers as $key => $ans) {
            $ans->id = $key;
            $state->options->answers[$key] = clone($ans);	
        }

        // create an array of answerids 
        $answerids = array_values(array_map(create_function('$val','return $val->id;'), $state->options->answers));
        $answerids = swapshuffle($answerids);
        $state->responses = $answerids;

        $this->loger('create_session_and_responses - END');
        return true;
    }

    /**
     * Restores the session data and most recent responses for the given state
     *
     * This function loads any session data associated with the question
     * session in the given state from the database into the state object.
     * In particular it loads the responses that have been saved for the given
     * state into the ->responses member of the state object.
     *
     * @return bool            Indicates success or failure.
     * @param object $question The question object for the question including any
     *                         question type specific information.
     * @param object $state    The saved state to load the session for. This
     *                         object should be updated to include the question
     *                         type specific session information and responses
     *                         (it is passed by reference).
     */
    function restore_session_and_responses(&$question, &$state) {
        // TODO unpack $state->responses[''], which has just been loaded from the
        // database field question_states.answer into the $state->responses array.
        $this->loger('restore_session_and_responses - START');
        		
        $state->responses = explode(',', $state->responses['']);
		
        // Place the questions into the state options keyed by code
        foreach ($question->options->answers as $answer) {
            $state->options->answers[$answer->code] = $answer;
        }		
        foreach ($state->options->answers as $key => $ans) {  
            $ans->id = $key;			
            $state->options->answers[$key] = clone($ans);
        }
		
        //set up feedback image(image, which is shown)
        foreach ($state->responses as $key => $response) {
            if($state->options->answers[$response]->orderposition == $key){						
                $state->options->correctposition[$key] = 1;						
            }else if ($state->options->answers[$response]->orderposition != $key){
                $state->options->correctposition[$key] = 0;					
            }	
        }
                
        $this->loger('restore_session_and_responses - END');
        return true;
    }

    /**
     * Saves the session data and responses for the given question and state
     *
     * This function saves the question type specific session data from the
     * state object to the database. In particular for most question types it saves the
     * responses from the ->responses member of the state object. The question type
     * non-specific data for the state has already been saved in the question_states
     * table and the state object contains the corresponding id and
     * sequence number which may be used to index a question type specific table.
     *
     * @return bool            Indicates success or failure.
     * @param object $question The question object for the question including
     *                         the question type specific information.
     * @param object $state    The state for which the question type specific
     *                         data and responses should be saved.
     */
    function save_session_and_responses(&$question, &$state) {
        // TODO package up the students response from the $state->responses
        // array into a string and save it in the question_states.answer field.
        $responses = '';

        $this->loger('save_session_and_responses - START');
        
        $responses  = implode(',', $state->responses);		
        set_field('question_states', 'answer', $responses, 'id', $state->id);
        
        $this->loger('save_session_and_responses - END');
        
        return true;
        //return set_field('question_states', 'answer', $responses, 'id', $state->id);
    }

    /**
     * Prints the question  grading details, content,
     * feedback and interactions
     *
     * This function prints the question including 
     * grading details, content for the question, any feedback for the previously
	 * submitted responses and the interactions.
	 *
     * @param object $question The question to be rendered. Question type
     *                         specific information is included. The
     *                         maximum possible grade is in ->maxgrade. The name
     *                         prefix for any named elements is in ->name_prefix.
     * @param object $state    The state to render the question in. The grading
     *                         information is in ->grade, ->raw_grade and
     *                         ->penalty. The current responses are in
     *                         ->responses. This is an associative array (or the
     *                         empty string or null in the case of no responses
     *                         submitted). The last graded state is in
     *                         ->last_graded (hence the most recently graded
     *                         responses are in ->last_graded->responses). The
     *                         question type specific information is also
     *                         included.
     * @param object $cmoptions
     * @param object $options  An object describing the rendering options.
    */
    function print_question_formulation_and_controls(&$question, &$state, $cmoptions, $options) {
        global $CFG;
        $this->loger('print_question_formulation_and_controls - START');

        $contextid = get_field('question_categories', 'contextid', 'id', $question->category); 
        $context = get_context_instance_by_id($contextid);

//        $context = $this->get_context_by_category_id($question->category);
        
        $answers = $state->options->answers;

        // Print formulation - vypisanie otazky
        $questiontext = $this->format_text($question->questiontext,$question->questiontextformat, $cmoptions);
		
        //prepare answers for html select
        $possible_answers = array();
        foreach ($state->options->answers as $answer) {
            $possible_answers[$answer->id] = strip_tags(format_string($answer->answertext, false));
        }
		
        // Print each answer in a separate row	
        foreach ($state->responses as $key => $aid) {            
            $answer = &$answers[$aid];
            $name   = "name=\"{$question->name_prefix}{$aid}\"";
            $menuname = $question->name_prefix.$key;
            //$response = isset($state->responses[$key]) ? $state->responses[$key] : $aid;
            $response = $state->responses[$key];
			            
            $a = new stdClass;
            $a->id   = $question->name_prefix . $aid;
            $a->class = '';
            $a->feedbackimg = '';
            $a->nameprefix = "\"{$question->name_prefix}\"";
            // Print the control            
            $a->control = choose_from_menu($possible_answers, $menuname, $response, 'choose', '','0', true); //6 parameter musim dat na 'true', aby vratil skript a netlacil ho na vystup. Pretu su tam defaultne parametre na 4 a 5 pozicii
            //$a->control = html_writer::select($possible_answers, $menuname, $response);            
            //print feedback image
            if (($options->feedback) || $options->correct_responses) {
                $a->feedbackimg = question_get_feedback_image($state->options->correctposition[$key], $options->feedback);
            }
    		
            $anss[] = clone($a);
        }     
		   
        include("$CFG->dirroot/question/type/it4kt_order/display.html");
        
        $this->loger('print_question_formulation_and_controls - END');        
    }

    /**
     * Performs response processing and grading
     *
     * This function performs response processing and grading and updates
     * the state accordingly.
     * @return boolean         Indicates success or failure.
     * @param object $question The question to be graded. Question type
     *                         specific information is included.
     * @param object $state    The state of the question to grade. The current
     *                         responses are in ->responses. The last graded state
     *                         is in ->last_graded (hence the most recently graded
     *                         responses are in ->last_graded->responses). The
     *                         question type specific information is also
     *                         included. The ->raw_grade and ->penalty fields
     *                         must be updated. The method is able to
     *                         close the question session (preventing any further
     *                         attempts at this question) by setting
     *                         $state->event to QUESTION_EVENTCLOSEANDGRADE
     * @param object $cmoptions
     */
    function grade_responses(&$question, &$state, $cmoptions) {
        // TODO assign a grade to the response in state.
        $this->loger('grade_responses - START');

    	$responses    = $state->responses;
        $totalgrade = count($responses);
        $sumgrade = 0;
        
        foreach ($responses as $key => $response) {	
            if($state->options->answers[$response]->orderposition == $key){						
                $sumgrade += 1;					
                $state->options->correctposition[$key] = 1;	
            }else if ($state->options->answers[$response]->orderposition != $key){
                $sumgrade += 0;						
                $state->options->correctposition[$key] = 0;
            }
        }	
        
        $state->raw_grade = $sumgrade/$totalgrade;
        // Make sure we don't assign negative or too high marks
        $state->raw_grade = min(max((float) $state->raw_grade,0.0), 1.0) * $question->maxgrade;

        // Apply the penalty for this attempt
        $state->penalty = $question->penalty * $question->maxgrade;

        // mark the state as graded
        $state->event = ($state->event ==  QUESTION_EVENTCLOSE) ? QUESTION_EVENTCLOSEANDGRADE : QUESTION_EVENTGRADE;
        
        $this->loger('grade_responses - END');
        
        return true;
    }

    /**
     * Compares two question states for equivalence of the student's responses
     *
     * The responses for the two states must be examined to see if they represent
     * equivalent answers to the question by the student. This method will be
     * invoked for each of the previous states of the question before grading
     * occurs. If the student is found to have already attempted the question
     * with equivalent responses then the attempt at the question is ignored;
     * grading does not occur and the state does not change. Thus they are not
     * penalized for this case.
     * @return boolean
     * @param object $question  The question for which the states are to be
     *                          compared. Question type specific information is
     *                          included.
     * @param object $state     The state of the question. The responses are in
     *                           ->responses. This is the only field of $state
     *                           that it is safe to use.
     * @param object $teststate The state whose responses are to be
     *                          compared. The state will be of the same age or
     *                          older than $state. If possible, the method should
     *                          only use the field $teststate->responses, however
     *                          any field that is set up by restore_session_and_responses
     *                          can be used.
     */
    function compare_responses($question, $state, $teststate) {
        // TODO write the code to return two different student responses, and
        // return two if the should be considered the same.
        $this->loger('compare_responses - START');
        
    	foreach ($state->responses as $key=>$sr) {
            if (empty($teststate->responses[$key])) {
                if (!empty($state->responses[$key])) {
                    return false;
                }
            } else if ($state->responses[$key] != $teststate->responses[$key]) {
                return false;
            }
        }
        
        $this->loger('compare_responses - END');
        return false;
    }

    /**
     * Checks whether a response matches a given answer, taking the tolerance
     * and units into account. Returns a true for if a response matches the
     * answer, false if it doesn't.
     */
    function test_response(&$question, &$state, $answer) {
        // TODO if your code uses the question_answer table, write a method to
        // determine whether the student's response in $state matches the
        // answer in $answer.
        $this->loger('test_response - START');

        $this->loger('test_response - END');
        return false;
    }

    function check_response(&$question, &$state){
        // TODO
        $this->loger('check_response - START');

        $this->loger('check_response - END');
        return false;
    }

    /**
     * Returns an array of values which will give full marks if graded as
     * the $state->responses field
     *
     * The correct answer to the question in the given state, or an example of
     * a correct answer if there are many, is returned. This is used by some question
     * types in the {@link grade_responses()} function but it is also used by the
     * question preview screen to fill in correct responses.
     * @return mixed           A response array giving the responses corresponding
     *                         to the (or a) correct answer to the question. If there is
     *                         no correct answer that scores 100% then null is returned.
     * @param object $question The question for which the correct answer is to
     *                         be retrieved. Question type specific information is
     *                         available.
     * @param object $state    The state of the question, for which a correct answer is
     *                         needed. Question type specific information is included.
     */
    function get_correct_responses(&$question, &$state) {
        // TODO
        $this->loger('get_correct_responses - START');
        
        $responses = array();
        $i = 0;
        foreach ($question->options->answers as $answer) {
            $responses[$i++] = (string) $answer->code;			   
        }
        $state->responses = $responses;        
        
        $this->loger('get_correct_responses - END');
        return empty($responses) ? null : $responses;  
    }

    /**
     * Return an array of values with the texts for all possible responses stored
     * for the question
     *
     * All answers are found and their text values isolated
     * @return object          A mixed object
     *             ->id        question id. Needed to manage random questions:
     *                         it's the id of the actual question presented to user in a given attempt
     *             ->responses An array of values giving the responses corresponding
     *                         to all answers to the question. Answer ids are used as keys.
     *                         The text and partial credit are the object components
     * @param object $question The question for which the answers are to
     *                         be retrieved. Question type specific information is
     *                         available.
     */
    function get_all_responses(&$question, &$state) {
        $result = new stdClass;
        // TODO

        $this->loger('get_all_responses - START');

        $answers = array();
        if (is_array($question->options->answers)) {
            foreach ($question->options->answers as $aid => $answer) {
                if ($answer->answertext) {
                    $r = new stdClass;
                    $r->answer = $answer->answertext;
                    $r->credit = 1;
                    $answers[$aid] = $r;
                }
            }
        }
        $result = new stdClass;
        $result->id = $question->id;
        $result->responses = $answers;
        
        
        $this->loger('get_all_responses - END');
        return $result;
    }

    /**
     * Return the actual response to the question in a given state
     * for the question. Text is not yet formatted for output.
     *
     * @return mixed           An array containing the response or reponses (multiple answer, match)
     *                         given by the user in a particular attempt.
     * @param object $question The question for which the correct answer is to
     *                         be retrieved. Question type specific information is
     *                         available.
     * @param object $state    The state object that corresponds to the question,
     *                         for which a correct answer is needed. Question
     *                         type specific information is included.
     */
    function get_actual_response($question, $state) {
        // TODO
        $responses = '';
        $this->loger('get_actual_response - START');
        
        $results = array();
        $responses = $state->responses;
        foreach ($responses as $ind => $code) {
            $results[$ind] =  $state->options->answers[$responses[$ind]]->answertext;
        }
		
                
        $this->loger('get_actual_response - END');
        return $results;
    }

    /**
     * Backup the data in the question
     *
     * This is used in question/backuplib.php
     */
    function backup($bf,$preferences,$question,$level=6) {
        $status = true;

        // TODO write code to backup an instance of your question type.
        $this->loger('backup - START');

        $this->loger('backup - END');

        return $status;
    }

    /**
     * Restores the data in the question
     *
     * This is used in question/restorelib.php
     */
    function restore($old_question_id,$new_question_id,$info,$restore) {
        $status = true;

        // TODO write code to restore an instance of your question type.
        $this->loger('restore - START');

        $this->loger('restore - END');

        return $status;
    }


	/**
     * Imports question from the Moodle XML format
     *
     * Imports question using information from extra_question_fields function
     * If some of you fields contains id's you'll need to reimplement this
     */
    function import_from_xml($data, $question, $format, $extra=null) {
        $this->loger('import_from_xml - START');

        if (!array_key_exists('@', $data)) {
            return false;
        }
        if (!array_key_exists('type', $data['@'])) {
            return false;
        }
        if ($data['@']['type'] == 'it4kt_order') {
            $question = $format->import_headers($data);

            // header parts particular to ordering
            $question->qtype = 'it4kt_order';
            
            // get file
            $filexml = $format->getpath($data, array('#', 'file'), array());
            $question->xmlfile = $this->import_files_to_draft_file_area($format, $filexml);
			                
            $this->loger('import_from_xml - END - TRUE');
            return $question;
        }
        
        $this->loger('import_from_xml - END - FALSE');
        return false;
    }

    /**
     * Export question to the Moodle XML format
     *
     * Export question using information from extra_question_fields function
     * If some of you fields contains id's you'll need to reimplement this
     */
    function export_to_xml($question, $format, $extra) {
        $this->loger('export_to_xml - START');

        $expout = '';
        $files = get_area_files($question->id);
//        $fs = get_file_storage();
//        $contextid = $question->contextid;
//        $files = $fs->get_area_files($contextid, 'qtype_mborder', 'question', $question->id);
        
        $expout .= $format->write_files($files);
        
        $this->loger('export_to_xml - END');
        return $expout;
    }
    
    /************************************************************************/
    /********************** Pridane pomocne funkcie *************************/
    /************************************************************************/
    /**
     * Create a draft files area, import files into it and return the draft item id.
     * @param qformat_xml $format
     * @param array $xml an array of <file> nodes from the the parsed XML.
     * @return integer draftitemid
     */
    function import_files_to_draft_file_area($format, $files) {
        global $CFG,$COURSE,$USER;
        $this->loger('import_files_to_draft_file_area: START');
        foreach ($files as $file) {
            
            $filename = $file['@']['name'];
            
            $questiondir = $CFG->dataroot . '/question/' . $COURSE->id . '/'. $USER->id . '/draft/' . $filename;
            $xmlfile->encoding = $file['@']['encoding'];
            $xmlfile->content = $file['#'];
            $content = $this->decode_file($xmlfile);    
            //Ulozenie suboru
            $this->save_file($questiondir, $content);
        }
        
        $this->loger('import_files_to_draft_file_area: END');
        return $filename;
    }


    /**
     * Loads directory of files for question type identified by question ID.
     */
    function get_area_files($questionid) {
        global $CFG,$COURSE;
        $this->loger('get_area_files - START');
        $questiondir = $CFG->dataroot . '/question/' . $COURSE->id . '/' . $questionid;

        $questiondir1 = make_upload_directory($questiondir);    
        $this->loger('get_area_files - END - reading directory from: \'' . $questiondir1 . '\'');
        return get_directory_list($questiondir1);
    }

    /**
     * Save the file belonging to one text field.
     *
     * @param array $field the data from the form (or from import). This will
     *      normally have come from the formslib editor element, so it will be an
     *      array with keys 'text', 'format' and 'itemid'. However, when we are
     *      importing, it will be an array with keys 'text', 'format' and 'files'
     * @param object $context the context the question is in.
     * @param string $component indentifies the file area question.
     * @param string $filearea indentifies the file area questiontext, generalfeedback,answerfeedback.
     * @param integer $itemid identifies the file area.
     *
     * @return string the text for this field, after files have been processed.
     */
    function import_or_save_files($field, $context, $component, $filearea, $itemid) {
        global $CFG,$COURSE, $USER;
        if (!empty($field['itemid'])) {
            // This is the normal case. We are safing the questions editing form.

            //Nacitanie obsahu suboru z draft oblasti
            $questiondir_draft = $CFG->dataroot . '/question/' . $COURSE->id . '/'. $USER->id . '/draft/';            
            $filename_draft =  $questiondir_draft . $field['itemid'];
            $this->loger('import_or_save_files: saving question from editing form from file \'' . $filename_draft . '\'');
            $content = $this->get_file_content($filename_draft);
            $this->loger('import_or_save_files: saving question from editing form from file \'' . $filename_draft . '\'');
                        
            //Ulozenie suboru
            $questiondir = $CFG->dataroot . '/question/' . $COURSE->id . '/' . $itemid . '/';
            $filename =  $questiondir . $itemid;
            $this->save_file($filename, $content);
        } else if (!empty($field['files'])) {
            $this->loger('import_or_save_files: saving question from import');
        // This is the case when we are doing an import.
            foreach ($field['files'] as $file) {
                $this->import_file($context, $component,  $filearea, $itemid, $file);
            }
        }
        
        $this->loger('import_or_save_files: return text \'' . trim($field['text']) .'\'');
        return trim($field['text']);
    }     

    function import_file($context, $component, $filearea, $itemid, $file) {
       global $COURSE;
        $questiondir = $CFG->dataroot . '/question/' . $COURSE->id . '/' .  $itemid . '/' . $itemid;
        $file1 = fopen($questiondir, "w");
        fwrite($file1, $this->decode_file($file));
        fclose($file1);            
    }
    
    /**
     * Save file.
     *
     * @param $filename is full path including file name
     * @param $content id file content
     */
    function save_file($filename, $content)
    {
        $this->loger('save_file: saving file \'' . $filename .'\'' );
        $dirname = dirname($filename);
        if (!is_dir($dirname))
        {
            mkdir($dirname, 0777, true);
        }
        
        $file = fopen($filename, "w");
        fwrite($file, $content);
        fclose($file);                
    }

    
    /**
     * Logger.
     *
     * @param $filename is full path including file name
     * @param $content id file content
     */
    function loger($log)
    {
        global $CFG;
        
        $logfile = $CFG->dataroot . '/log/order.txt';
        $dirname = dirname($logfile);
        if (!is_dir($dirname))
        {
            mkdir($dirname, 0777, true);
        }
        
        $file = fopen($logfile, "a");
        fwrite($file, date("d.m.Y H:i:s", time()) . ': ' . $log . "\n");
        fclose($file);                
    }

    /**
     * Save file.
     *
     * @param $filename is full path including file name
     * @param $content id file content
     */
    function get_file_content($filename)
    {        
        //return file_get_contents($filename);
        $this->loger('get_file_content: Read from file \'' . $filename . '\'');
        
        $file = fopen($filename, "r");
        $this->loger('get_file_content: fopen with filesize: ' . filesize($filename));
        $content = fread($file, filesize($filename));
        $this->loger('get_file_content: fread');
        fclose($file);                
        
        return $content;
    }
    
    function decode_file($file) {
        switch ($file->encoding) {
        case 'base64': return base64_decode($file->content);
        default:
            return $file->content;
        }
    }     
    /************************************************************************/
    
}

// Register this question type with the system.
question_register_questiontype(new it4kt_order_qtype());
?>
